@@ -1,4 +1,4 @@
-/* armor.c - Armor/Text handling for data
+/* armor.c - Armor filters
  *        Copyright (C) 2001, 2002 Timo Schulz
  *        Copyright (C) 1998-2001 Free Software Foundation, Inc.
  *
@@ -20,7 +20,6 @@
  *
  *
  * ChangeLog for basic BASE64 code (base64_encode, base64_decode):
- *
  * Original author: Eric S. Raymond (Fetchmail)
  * Heavily modified by Brendan Cully <brendan@kublai.com> (Mutt)
  * Modify the code for generic use by Timo Schulz <twoaday@freakmail.de>
@@ -378,7 +377,14 @@ search_header (const char *buf, const char **array)
     }
   return -1;
 }
-  
+
+
+const char *
+_cdk_armor_get_lineend (void)
+{
+  return LF;
+}
+
 
 int
 armor_decode (void * opaque, FILE *in, FILE *out)
@@ -535,7 +541,7 @@ cdk_file_armor (CDK_HD hd, const char *file, const char *output)
   return rc;
 }
 
-#include "stream.h"
+
 /**
  * cdk_file_dearmor: Remove ASCII armor from a file.
  * @file: Name of the file to unprotect.
