@@ -23,12 +23,17 @@
 # include <config.h>
 #endif
 #include <stdio.h>
+#include <sys/stat.h>
 
 #include "opencdk.h"
 #include "main.h"
 #include "filters.h"
 
 
+static void (*progress_cb)(void * hd, unsigned off, unsigned len);
+static void * progress_cb_value = NULL;
+
+
 int
 _cdk_cipher_test_algo (int algo)
 {
@@ -110,6 +115,7 @@ write_header (cipher_filter_s * cfx, CDK_STREAM out)
     }
 
   init_packet (&pkt);
+  pkt.old_ctb = cfx->dek->rfc1991? 1 : 0;
   pkt.pkttype = use_mdc ? CDK_PKT_ENCRYPTED_MDC : CDK_PKT_ENCRYPTED;
   pkt.pkt.encrypted = &ed;
   rc = cdk_pkt_build (out, &pkt);
@@ -154,14 +160,19 @@ write_header (cipher_filter_s * cfx, CDK_STREAM out)
 static int
 cipher_encode2 (void * opaque, FILE * in, FILE * out)
 {
+  struct stat statbuf;
   cipher_filter_s * cfx = opaque;
   byte buf[4096], pktdata[22];
   int algo = GCRY_MD_SHA1, dlen = gcry_md_get_algo_dlen (algo);
-  int rc = 0, nread;
+  int rc = 0, nread, len;
 
   if (!cfx || !in || !out)
     return CDK_Inv_Value;
 
+  if (fstat (fileno (in), &statbuf))
+    return CDK_File_Error;
+  len = statbuf.st_size;
+  
   while (!feof (in))
     {
       nread = fread (buf, 1, sizeof buf -1, in);
@@ -175,6 +186,8 @@ cipher_encode2 (void * opaque, FILE * in, FILE * out)
           rc = CDK_Gcry_Error;
           break;
         }
+      if (progress_cb)
+        progress_cb (progress_cb_value, ftell (in), len);
       fwrite (buf, 1, nread, out);
     }
   wipemem (buf, sizeof buf);
@@ -194,6 +207,8 @@ cipher_encode2 (void * opaque, FILE * in, FILE * out)
       cfx->mdc_hash = NULL;
       fwrite (pktdata, 1, 22, out);
     }
+  gcry_cipher_close (cfx->cipher_hd);
+  cfx->cipher_hd = NULL;
   return rc;
 }
 
@@ -252,6 +267,25 @@ read_header (cipher_filter_s * cfx, FILE * in)
 }
 
 
+static int
+finalize_mdc (GCRY_MD_HD md, const byte * buf, size_t nread)
+{
+  byte mdcbuf[20];
+  int rc = 0;
+  
+  if (buf[nread - 22] == 0xd3 && buf[nread - 21] == 0x14)
+    {
+      gcry_md_write (md, buf, nread - 20);
+      gcry_md_final (md);
+      memcpy (mdcbuf, gcry_md_read (md, 0), 20);
+      if (memcmp (mdcbuf, buf + nread - 20, 20))
+          rc = CDK_Bad_MDC;
+      return rc;
+    }
+  return CDK_Inv_Packet;
+}
+
+  
 static int
 cipher_decode2 (void * opaque, FILE * in, FILE * out)
 {
@@ -273,28 +307,38 @@ cipher_decode2 (void * opaque, FILE * in, FILE * out)
           rc = CDK_Gcry_Error;
           break;
         }
-      if (cfx->mdc_hash)
+      if (feof (in) && cfx->mdc_hash)
+        rc = finalize_mdc (cfx->mdc_hash, buf, nread);
+      else if (cfx->mdc_hash)
         gcry_md_write (cfx->mdc_hash, buf, nread);
       fwrite (buf, 1, nread, out);
     }
+  gcry_cipher_close (cfx->cipher_hd);
+  cfx->cipher_hd = NULL;
+  if (cfx->mdc_hash)
+    {
+      gcry_md_close (cfx->mdc_hash);
+      cfx->mdc_hash = NULL;
+    }
   wipemem (buf, sizeof buf);
   return rc;
 }
 
 
 int
-cipher_decode (void *opaque, FILE * in, FILE * out)
+cipher_decode (void * opaque, FILE * in, FILE * out)
 {
   cipher_filter_s * cfx = opaque;
   int rc;
-    
+
+  _cdk_log_debug ("cipher filter: decode\n");
+  
   if (!cfx || !in || !out)
     return CDK_Inv_Value;
     
   rc = read_header (cfx, in);
   if (!rc)
     rc = cipher_decode2 (cfx, in, out);
-
   return rc;
 }
     
@@ -306,6 +350,8 @@ cipher_encode (void * opaque, FILE * in, FILE * out)
   CDK_STREAM si, so;
   int rc;
 
+  _cdk_log_debug ("cipher filter: encode\n");
+  
   if (!cfx || !in || !out)
     return CDK_Inv_Value;
 
@@ -328,3 +374,13 @@ cipher_encode (void * opaque, FILE * in, FILE * out)
 
   return rc;
 }
+
+
+void
+cdk_set_progress_handler (void (*cb)(void * hd, unsigned off, unsigned size),
+                          void * cb_value)
+{
+  progress_cb = cb;
+  progress_cb_value = cb_value;
+}
+
