@@ -61,6 +61,7 @@ struct cdk_handle_s
     unsigned compress:1;
     unsigned mdc:1;
     unsigned compat:1;
+    unsigned rfc1991:1;
   } opt;
   struct 
   {
@@ -119,7 +120,7 @@ struct cdk_keydb_search_s
 
 struct cdk_key_list_s
 {
-  CDK_KEYLIST next;
+  struct cdk_key_list_s * next;
   union
   {
     cdkPKT_public_key * pk;
@@ -135,6 +136,7 @@ struct cdk_dek_s
   int algo;
   int keylen;
   int use_mdc;
+  unsigned rfc1991:1;
   unsigned char key[32]; /* 256-bit */
 };
 
