@@ -50,6 +50,7 @@ struct mainproc_ctx_s
   } sig;
   unsigned reset:1;
   unsigned eof_seen:1;
+  unsigned key_seen:1;
   char * file; /* for detached signatures */
 };
 typedef struct mainproc_ctx_s *CTX;
@@ -69,6 +70,16 @@ write_marker_packet (CDK_STREAM out)
 }
 
 
+static void
+plaintext_set_rfc1991 (CDK_STREAM out)
+{
+  plaintext_filter_s * pfx;
+  pfx = _cdk_stream_get_opaque (out, fPLAINTEXT);
+  if (pfx)
+    pfx->rfc1991 = 1;
+}
+
+
 static int
 sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
 {
@@ -102,6 +113,12 @@ sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
       goto fail;
     }  
 
+  if (hd->opt.rfc1991)
+    {
+      hd->dek->rfc1991 = 1;
+      goto start; /* skip the pkt_symkey_enc packet. */
+    }
+  
   cdk_stream_set_cache (out, 1);
   if (hd->opt.compat)
     write_marker_packet (out);
@@ -124,12 +141,15 @@ sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
     goto fail;
   cdk_stream_set_cache (out, 0);
 
+start:
   if (hd->opt.armor)
     cdk_stream_set_armor_flag (out, 0);
   cdk_stream_set_cipher_flag (out, hd->dek, hd->opt.mdc);
   if (hd->opt.compress)
     cdk_stream_set_compress_flag (out, hd->compress_algo);
   cdk_stream_set_literal_flag (out, 0, _cdk_stream_get_fname (inp));
+  if (hd->opt.rfc1991)
+    plaintext_set_rfc1991 (out);
 
   rc = cdk_stream_kick_off (inp, out);
 
@@ -139,6 +159,32 @@ fail:
 }
 
 
+static int
+use_rfc1991_format (CDK_HD hd, CDK_KEYLIST kl)
+{
+  CDK_KEYLIST l;
+  
+  if (hd->opt.rfc1991)
+    return 1;
+  for (l = kl; l; l = l->next)
+    {
+      if (l->type == CDK_PKT_PUBLIC_KEY && l->key.pk->version == 3)
+        return 1;
+    }
+  return 0;
+}
+
+
+static int
+select_cipher_algo (int rfc1991, CDK_KEYLIST kl)
+{
+  int pgp2 = _cdk_is_idea_available ();
+  int def_cipher = pgp2 && rfc1991? GCRY_CIPHER_IDEA : GCRY_CIPHER_CAST5;
+  
+  return rfc1991? def_cipher : cdk_pklist_select_algo (kl);
+} 
+  
+
 /**
  * cdk_stream_encrypt: Encrypt a stream.
  * @hd: Handle
@@ -154,7 +200,8 @@ cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
                     CDK_STREAM inp, CDK_STREAM out)
 {
   CDK_KEYLIST pkl = NULL;
-  int cipher_algo;
+  int cipher_algo, compress_algo = 0;
+  int use_rfc1991 = 0;
   int rc = 0;
 
   if (!hd || !inp || !out)
@@ -167,7 +214,8 @@ cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
   if (rc)
     return rc;
 
-  cipher_algo = cdk_pklist_select_algo (pkl);
+  use_rfc1991 = use_rfc1991_format (hd, pkl);
+  cipher_algo = select_cipher_algo (use_rfc1991, pkl);
   cdk_free (hd->dek);
   rc = cdk_dek_new (&hd->dek, cipher_algo);
   if (rc)
@@ -175,11 +223,14 @@ cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
       cdk_pklist_release (pkl);
       return rc;
     }
-  
-  hd->dek->use_mdc = cdk_pklist_use_mdc (pkl);
+  compress_algo = use_rfc1991? CDK_COMPRESS_ZIP : hd->compress_algo;
+
+  if (!hd->opt.rfc1991 && !hd->opt.compat)
+    hd->dek->use_mdc = cdk_pklist_use_mdc (pkl);
+  hd->dek->rfc1991 = use_rfc1991;
 
   cdk_stream_set_cache (out, 1);
-  if (hd->opt.compat)
+  if (hd->opt.compat && !hd->opt.rfc1991)
     write_marker_packet (out);
   
   rc = cdk_pklist_encrypt (pkl, hd->dek, out);
@@ -194,8 +245,10 @@ cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
     cdk_stream_set_armor_flag (out, 0);
   cdk_stream_set_cipher_flag (out, hd->dek, 0);
   if (hd->opt.compress)
-    cdk_stream_set_compress_flag (out, hd->compress_algo);
+    cdk_stream_set_compress_flag (out, compress_algo);
   cdk_stream_set_literal_flag (out, 0, _cdk_stream_get_fname (inp));
+  if (hd->dek->rfc1991)
+    plaintext_set_rfc1991 (out);
 
   rc = cdk_stream_kick_off (inp, out);
 
@@ -204,25 +257,26 @@ cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
 }
 
 
+/**
+ * cdk_file_encrypt: Encrypt a file.
+ * @hd: Handle
+ * @remusr: List of recipient
+ * @file: Input file
+ * @output: Output file
+ *
+ **/
 int
 cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
                   const char * file, const char * output)
 {
-  CDK_STREAM inp, out;
+  CDK_STREAM inp = NULL, out = NULL;
   int rc;
   
   rc = cdk_stream_open (file, &inp);
-  if (rc)
-    return rc;
-
-  rc = cdk_stream_new (output, &out);
-  if (rc)
-    {
-      cdk_stream_close (inp);
-      return rc; 
-    }
-
-  rc = cdk_stream_encrypt (hd, remusr, inp, out);
+  if (!rc)
+    rc = cdk_stream_new (output, &out);
+  if (!rc)
+    rc = cdk_stream_encrypt (hd, remusr, inp, out);
 
   cdk_stream_close (inp);
   cdk_stream_close (out);
@@ -315,6 +369,7 @@ handle_symkey_enc (CTX c, CDK_PACKET * pkt)
 
   assert (pkt->pkttype == CDK_PKT_SYMKEY_ENC);
 
+  c->key_seen = 1;
   if (c->dek)
     return 0; /* we already decrypted the session key */
   
@@ -340,6 +395,7 @@ handle_pubkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
 
   assert (pkt->pkttype == CDK_PKT_PUBKEY_ENC);
 
+  c->key_seen = 1;
   enc = pkt->pkt.pubkey_enc;
   write_status (hd, CDK_CB_PUBKEY_ENC, "%08lX%08lX %d %d",
                 enc->keyid[0], enc->keyid[1], enc->pubkey_algo,
@@ -363,17 +419,57 @@ handle_pubkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
 }
 
 
+static int
+rfc1991_get_sesskey (CDK_DEK * r_dek)
+{
+  char * pw;
+  CDK_S2K_HD s2k;
+
+  if (!r_dek)
+    return CDK_Inv_Value;
+  
+  pw = _cdk_passphrase_get ("Enter Passphrase: ");
+  if (!pw)
+    return CDK_Out_Of_Core;
+  s2k = cdk_calloc (1, sizeof *s2k);
+  if (!s2k)
+    {
+      _cdk_passphrase_free (pw);
+      return CDK_Out_Of_Core;
+    }
+  s2k->mode = 0;
+  s2k->hash_algo = GCRY_MD_MD5;
+  *r_dek = cdk_passphrase_to_dek (GCRY_CIPHER_IDEA, s2k, 0, pw);
+  _cdk_passphrase_free (pw);
+  cdk_free (s2k);
+  return *r_dek? 0 : CDK_Error_No_Key;
+}
+
+  
 static int
 handle_encrypted (CTX c, CDK_PACKET *pkt, int use_mdc)
 {
   cdkPKT_encrypted * enc;
+  int pgp2_compat = _cdk_is_idea_available ();
   int rc = 0;
 
   assert (pkt->pkttype == CDK_PKT_ENCRYPTED
           || pkt->pkttype == CDK_PKT_ENCRYPTED_MDC);
 
   if (!c->dek)
-    return CDK_Error_No_Key;
+    {
+      if (!pgp2_compat)
+        return CDK_Error_No_Key;
+      else if (!c->key_seen)
+        {
+          _cdk_log_debug ("RFC1991 message was detected.\n");
+          rc = rfc1991_get_sesskey (&c->dek);
+          if (rc)
+            return rc;
+        }
+      else
+        return CDK_Error_No_Key;
+    }
   
   enc = pkt->pkt.encrypted;
   set_reset (c, enc->buf);
@@ -381,6 +477,8 @@ handle_encrypted (CTX c, CDK_PACKET *pkt, int use_mdc)
   rc = cdk_stream_read (enc->buf, NULL, 0);
   if (!rc)
     c->s = enc->buf;
+  else
+    rc = _cdk_stream_get_errno (enc->buf);
   return rc;
 }
 
@@ -396,7 +494,8 @@ handle_compressed (CTX c, CDK_PACKET * pkt)
   zip = pkt->pkt.compressed;
   cdk_stream_set_compress_flag (c->s, zip->algorithm);
   rc = cdk_stream_read (c->s, NULL, 0);
-
+  if (rc)
+    rc = _cdk_stream_get_errno (c->s);
   return rc;
 }
 
@@ -410,7 +509,6 @@ handle_onepass_sig (CTX c, CDK_PACKET * pkt)
   c->sig.md = gcry_md_open (c->sig.digest_algo, 0);
   if (!c->sig.md)
     return CDK_Gcry_Error;
-  
   return 0;
 }
 
@@ -434,7 +532,6 @@ handle_plaintext (CTX c, CDK_PACKET * pkt, CDK_STREAM out)
         pfx->md = c->sig.md;
     }
   rc = cdk_stream_kick_off (c->s, out);
-  
   return rc;
 }
 
@@ -484,7 +581,6 @@ handle_signature (CDK_HD hd, CTX c, CDK_PACKET * pkt)
   if (!rc)
     _cdk_log_debug ("Good Signature from %08lX%08lX (expired %d)\n",
                     keyid[0], keyid[1], c->sig.is_expired);
-  
   return rc;
 }
 
@@ -504,6 +600,7 @@ free_mainproc (CTX c)
         }
       cdk_free (c->file);
       cdk_free (c->dek);
+      cdk_free (c);
     }
 }
 
@@ -514,7 +611,7 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
   CDK_PACKET * pkt = NULL;
   CDK_KBNODE n = NULL, node;
   const char * s;
-  int rc = 0, npos;
+  int rc = 0, npos, with_mdc = 0;
 
   if (!hd || !c)
     return CDK_Inv_Value;
@@ -533,7 +630,8 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
       if (!pkt)
         return CDK_Out_Of_Core;
       rc = cdk_pkt_parse (inp, pkt);
-      _cdk_log_debug ("type=%d len=%d (%d)\n",pkt->pkttype, pkt->pktlen, rc);
+      _cdk_log_debug ("type=%d old_ctb=%d len=%d (%d)\n",
+                      pkt->pkttype, pkt->old_ctb, pkt->pktlen, rc);
       if (rc == CDK_EOF)
         c->eof_seen = 1;
       if (rc)
@@ -559,7 +657,8 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
           
         case CDK_PKT_ENCRYPTED_MDC: 
         case CDK_PKT_ENCRYPTED:
-          rc = handle_encrypted (c, pkt, pkt->pkttype ==CDK_PKT_ENCRYPTED_MDC);
+          with_mdc = pkt->pkttype == CDK_PKT_ENCRYPTED_MDC;
+          rc = handle_encrypted (c, pkt, with_mdc);
           _cdk_log_debug (" handle_encrypted (%d)\n", rc);
           if (!rc)
             inp = c->s;
@@ -595,6 +694,7 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
           break; /* handle it later */
 
         case CDK_PKT_MDC:
+          _cdk_log_debug ("MDC packet detected\n");
           break;
 
         default:
@@ -627,6 +727,7 @@ do_proc_packets (CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM out)
           break;
 
         default:
+          _cdk_log_debug ("skip packet type %d\n", pkt->pkttype);
           break;
         }
       if (rc)
@@ -649,8 +750,7 @@ proc_encryption_packets (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
   
   c = cdk_calloc (1, sizeof *c);
   if (!c)
-      return CDK_Out_Of_Core; 
-  
+      return CDK_Out_Of_Core;
   rc = do_proc_packets (hd, c, inp, out);
   free_mainproc (c);
   return rc;
@@ -690,24 +790,17 @@ _cdk_proc_signature_packets (CDK_HD hd, GCRY_MD_HD md,
 int
 cdk_file_decrypt (CDK_HD hd, const char *file, const char *output)
 {
-  CDK_STREAM inp, out;
+  CDK_STREAM inp = NULL, out = NULL;
   int rc;
   
   if (!file || !output)
     return CDK_Inv_Value;
 
   rc = cdk_stream_open (file, &inp);
-  if (rc)
-    return rc;
-
-  rc = cdk_stream_new (output, &out);
-  if (rc)
-    {
-      cdk_stream_close (inp);
-      return rc;
-    }
-
-  rc = proc_encryption_packets (hd, inp, out);
+  if (!rc)
+    rc = cdk_stream_new (output, &out);
+  if (!rc)
+    rc = proc_encryption_packets (hd, inp, out);
 
   cdk_stream_close (inp);
   cdk_stream_close (out);
