@@ -21,6 +21,12 @@
 #ifndef CDK_FILTERS_H
 #define CDK_FILTERS_H
 
+enum
+{
+  S_READ  = 0,
+  S_WRITE = 1
+};
+
 typedef struct
 {
   GCRY_CIPHER_HD cipher_hd;
@@ -52,6 +58,7 @@ typedef struct
 typedef struct
 {
   int mode;
+  unsigned rfc1991:1;
   char * filename;
   GCRY_MD_HD md;
 } plaintext_filter_s;
