@@ -31,6 +31,8 @@
 #include "main.h"
 #include "packet.h"
 
+#define is_deleted_kbnode(a)  ((a)->private_flag & 1)
+#define is_cloned_kbnode(a)   ((a)->private_flag & 2)
 
 #define USE_UNUSED_NODES 1
 static CDK_KBNODE unused_nodes;
@@ -86,16 +88,11 @@ cdk_kbnode_new (CDK_PACKET * pkt)
 }
 
 
-CDK_KBNODE
-cdk_kbnode_clone (CDK_KBNODE node)
+void
+_cdk_kbnode_clone (CDK_KBNODE node)
 {
-  CDK_KBNODE n = alloc_node ();
-  if (!n)
-    return NULL;
-  n->pkt = node->pkt;
-  n->private_flag = node->private_flag | 2; /* mark cloned */
-
-  return n;
+  if (node)
+    node->private_flag |= 2; /* mark cloned */
 }
 
 
@@ -132,7 +129,7 @@ cdk_kbnode_delete (CDK_KBNODE node)
  * Append NODE to ROOT.  ROOT must exist!
  */
 void
-cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node)
+_cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node)
 {
   CDK_KBNODE n1;
 
@@ -391,3 +388,68 @@ cdk_kbnode_get_packet (CDK_KBNODE node)
     return node->pkt;
   return NULL;
 }
+
+
+int
+cdk_kbnode_read_from_mem (CDK_KBNODE * ret_node,const byte * buf,size_t buflen)
+{
+  CDK_STREAM inp;
+  int rc;
+
+  if (!buflen)
+      return CDK_Inv_Value;
+
+  if (ret_node)
+    *ret_node = NULL;
+  
+  inp = cdk_stream_tmp_from_mem (buf, buflen);
+  if (!inp)
+      return CDK_Out_Of_Core;
+  
+  rc = cdk_keydb_get_keyblock (inp, ret_node);
+  if (rc == CDK_EOF)
+      rc = 0;
+
+  cdk_stream_close (inp);
+  return rc;
+}
+
+
+int
+cdk_kbnode_write_to_mem (CDK_KBNODE node, byte * buf, size_t * r_nbytes)
+{
+  CDK_KBNODE n;
+  CDK_STREAM s;
+  int rc = 0, len;
+  
+  s = cdk_stream_tmp ();
+  if (!s)
+    return CDK_Out_Of_Core;
+  
+  for (n = node; n; n = n->next)
+    {
+      if (n->pkt->pkttype != CDK_PKT_PUBLIC_KEY
+          && n->pkt->pkttype != CDK_PKT_PUBLIC_SUBKEY
+          && n->pkt->pkttype != CDK_PKT_SECRET_KEY
+          && n->pkt->pkttype != CDK_PKT_SECRET_SUBKEY
+          && n->pkt->pkttype != CDK_PKT_SIGNATURE
+          && n->pkt->pkttype != CDK_PKT_USER_ID)
+        continue;
+      rc = cdk_pkt_build (s, n->pkt);
+      if (rc)
+        break;
+    }
+  if (!rc)
+    {
+      cdk_stream_seek (s, 0);
+      len = cdk_stream_get_length (s);
+      if (*r_nbytes < len)
+        rc = CDK_Too_Short;
+      if (!rc)
+        *r_nbytes = cdk_stream_read (s, buf, len);
+    }
+  cdk_stream_close (s);
+  
+  return rc;
+}
+
