@@ -22,7 +22,10 @@
 # include <config.h>
 #endif
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#include <time.h>
+#include <ctype.h>
 
 #include "opencdk.h"
 #include "main.h"
@@ -167,9 +170,10 @@ cdk_keydb_get_ctx (int is_secret, int id)
 
 
 int
-cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, byte * data, size_t count)
+cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count)
 {
   CDK_KEYDB_HD hd;
+  char * name;
 
   hd = cdk_calloc (1, sizeof *hd);
   if (!hd)
@@ -180,8 +184,9 @@ cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, byte * data, size_t count)
     {
     case CDK_DBTYPE_ARMORED:
     case CDK_DBTYPE_KEYRING:
-      data[count] = '\0';
-      hd->name = strdup (data);
+      name = data;
+      name[count] = '\0';
+      hd->name = strdup (name);
       if (!hd->name)
         {
           cdk_free (hd);
@@ -191,14 +196,13 @@ cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, byte * data, size_t count)
       break;
       
     case CDK_DBTYPE_DATA:
-      hd->buf = cdk_stream_tmp ();
+      hd->buf = cdk_stream_tmp_from_mem (data, count);      
       if (!hd->buf)
         {
           cdk_free (hd);
           return CDK_Out_Of_Core;
         }
       hd->type = CDK_DBTYPE_DATA;
-      cdk_stream_write (hd->buf, data, count);
       break;
     }
   if (r_hd)
@@ -213,7 +217,7 @@ cdk_keydb_open (CDK_KEYDB_HD kdb, CDK_STREAM * ret_kr)
   int rc = 0;
   CDK_STREAM kr = NULL;
 
-  if (!kdb || !ret_kr)
+  if (!kdb)
     return CDK_Inv_Value;
 
   if (kdb->type == CDK_DBTYPE_DATA && kdb->buf)
@@ -238,7 +242,8 @@ cdk_keydb_open (CDK_KEYDB_HD kdb, CDK_STREAM * ret_kr)
       cdk_stream_close (kr);
       kr = NULL;
     }
-  *ret_kr = kr;
+  if (ret_kr)
+    *ret_kr = kr;
   return rc;
 }
 
@@ -308,7 +313,7 @@ find_by_fpr (CDK_KBNODE knode, CDK_DBSEARCH ks)
         }
     }
   return key_found;
-}          
+}
 
 
 static int
@@ -340,11 +345,9 @@ find_by_pattern (CDK_KBNODE knode, CDK_DBSEARCH ks)
         case CDK_DBSEARCH_SUBSTR:
           if (uidlen > 65536)
             break;
-          if (name
-              && strlen (ks->u.pattern) > uidlen)
+          if (name && strlen (ks->u.pattern) > uidlen)
             break;
-          if (name
-              && ascii_memistr (name, uidlen, ks->u.pattern))
+          if (name && _cdk_memistr (name, uidlen, ks->u.pattern))
             {
               key_found = 1;
               break;
@@ -429,7 +432,7 @@ cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
   int rc = 0;
   size_t pos = 0;
 
-  if (!kdb || !r_key || !ks)
+  if (!kdb || !ks)
     return CDK_Inv_Value;
 
   if (r_key)
@@ -482,8 +485,8 @@ cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
     }
 
   cdk_stream_close (kr);
-  *r_key = key_found? knode : NULL;
-
+  if (r_key)
+    *r_key = key_found? knode : NULL;
   return rc;
 }
 
@@ -590,7 +593,6 @@ _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid)
       if (kid[1] == keyid[1])
 	return node;
     }
-
   return NULL;
 }
 
@@ -611,7 +613,7 @@ cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key ** ret_sk,
   hd = cdk_keydb_get_ctx (1, -1);
   if (!hd)
     return CDK_Error_No_Keyring;
-  rc = cdk_keydb_get_bypattern (hd, name, &knode);
+  rc = cdk_keydb_search_auto (hd, name, &knode);
   if (rc)
     goto leave;
   if (alloced)
@@ -641,34 +643,34 @@ leave:
 
 
 int
-cdk_keydb_get_pk_byname (const char *name, cdkPKT_public_key ** ret_pk,
+cdk_keydb_get_pk_byname (const char * name, cdkPKT_public_key ** ret_pk,
 			 int alloced)
 {
-  CDK_KBNODE key, node = NULL;
+  CDK_KBNODE knode, node = NULL;
   cdkPKT_public_key *pk = NULL;
   CDK_KEYDB_HD hd;
   const char *s;
   int req_usage = 0;
   int rc = 0;
 
-  if (!name || !ret_pk)
+  if (!name)
     return CDK_Inv_Value;
 
   hd = cdk_keydb_get_ctx (0, -1);
   if (!hd)
     return CDK_Error_No_Keyring;
-  rc = cdk_keydb_get_bypattern (hd, name, &key);
+  rc = cdk_keydb_search_auto (hd, name, &knode);
   if (rc)
     goto leave;
   if (alloced)
     {
       req_usage = ret_pk[0]->pubkey_usage;
       cdk_free (ret_pk[0]);
-      node = keydb_find_byusage (key, req_usage, 1);
+      node = keydb_find_byusage (knode, req_usage, 1);
     }
   if (!node)
     {
-      node = cdk_kbnode_find (key, CDK_PKT_PUBLIC_KEY);
+      node = cdk_kbnode_find (knode, CDK_PKT_PUBLIC_KEY);
       if (!node)
 	{
 	  rc = CDK_Error_No_Key;
@@ -676,23 +678,24 @@ cdk_keydb_get_pk_byname (const char *name, cdkPKT_public_key ** ret_pk,
 	}
     }
   _cdk_copy_pubkey (&pk, node->pkt->pkt.public_key);
-  for (node = key; node; node = node->next)
+  for (node = knode; node; node = node->next)
     {
       if (node->pkt->pkttype == CDK_PKT_USER_ID)
 	{
 	  s = node->pkt->pkt.user_id->name;
-	  if (pk && !pk->uid && ascii_memistr (s, strlen (s), name))
+	  if (pk && !pk->uid && _cdk_memistr (s, strlen (s), name))
 	    {
 	      _cdk_copy_userid (&pk->uid, node->pkt->pkt.user_id);
 	      break;
 	    }
 	}
     }
-  cdk_kbnode_release (key);
+  cdk_kbnode_release (knode);
 
 leave:
   cdk_keydb_search_reset (hd);
-  *ret_pk = pk;
+  if (ret_pk)
+    *ret_pk = pk;
   return rc;
 }
 
@@ -793,11 +796,12 @@ keydb_merge_selfsig (CDK_KBNODE key)
   cdkPKT_user_id *uid = NULL;
   const byte *symalg = NULL, *hashalg = NULL, *compalg = NULL;
   size_t nsymalg = 0, nhashalg = 0, ncompalg = 0, n = 0;
+  int key_usage = 0, key_expire = 0;
 
   if (!key)
     return CDK_Inv_Value;
 
-  for (k = key; k && k->pkt->pkttype; k = k->next)
+  for (k = key; k; k = k->next)
     {
       if (k->pkt->pkttype == CDK_PKT_SIGNATURE
 	  && k->pkt->pkt.signature->sig_class == 0x13)
@@ -813,6 +817,17 @@ keydb_merge_selfsig (CDK_KBNODE key)
 	  s = cdk_subpkt_find (sig->hashed, SIGSUBPKT_FEATURES);
 	  if (s && s->size == 1 && s->d[0] & 0x01)
 	    uid->mdc_feature = 1;
+          s = cdk_subpkt_find (sig->hashed, SIGSUBPKT_KEY_EXPIRE);
+          if (s && s->size == 4)
+            key_expire = _cdk_buftou32 (s->d);
+          s = cdk_subpkt_find (sig->hashed, SIGSUBPKT_KEY_FLAGS);
+          if (s)
+            {
+              if (s->d[0] & 3) /* cert + sign data */
+                key_usage |= GCRY_PK_USAGE_SIGN;
+              if (s->d[0] & 12) /* encrypt comm. + storage */
+                key_usage |= GCRY_PK_USAGE_ENCR;
+            }
 	  s = cdk_subpkt_find (sig->hashed, SIGSUBPKT_PREFS_SYM);
 	  if (s)
 	    {
@@ -844,74 +859,67 @@ keydb_merge_selfsig (CDK_KBNODE key)
 	      n = 0;
 	      for (; nsymalg; nsymalg--, n++)
 		{
-		  uid->prefs[n].type = PREFTYPE_SYM;
+		  uid->prefs[n].type = CDK_PREFTYPE_SYM;
 		  uid->prefs[n].value = *symalg++;
 		}
 	      for (; nhashalg; nhashalg--, n++)
 		{
-		  uid->prefs[n].type = PREFTYPE_HASH;
+		  uid->prefs[n].type = CDK_PREFTYPE_HASH;
 		  uid->prefs[n].value = *hashalg++;
 		}
 	      for (; ncompalg; ncompalg--, n++)
 		{
-		  uid->prefs[n].type = PREFTYPE_ZIP;
+		  uid->prefs[n].type = CDK_PREFTYPE_ZIP;
 		  uid->prefs[n].value = *compalg++;
 		}
 	      /* end of list marker */
-	      uid->prefs[n].type = PREFTYPE_NONE;
+	      uid->prefs[n].type = CDK_PREFTYPE_NONE;
 	      uid->prefs[n].value = 0;
 	      uid->prefs_size = n;
 
 	      kbnode = cdk_kbnode_find_prev (key, k, CDK_PKT_PUBLIC_KEY);
-	      if (kbnode && uid->prefs && n)
+	      if (kbnode)
 		{
-		  cdkPKT_public_key *pk = kbnode->pkt->pkt.public_key;
-		  pk->prefs = _cdk_copy_prefs (uid->prefs);
-		  pk->prefs_size = n;
-		}
-	    }
-	}
+		  cdkPKT_public_key * pk = kbnode->pkt->pkt.public_key;
+                  if (uid->prefs && n)
+                    {
+                      pk->prefs = _cdk_copy_prefs (uid->prefs);
+                      pk->prefs_size = n;
+                    }
+                  if (key_expire)
+                    {
+                      pk->expiredate = pk->timestamp + key_expire;
+                      pk->has_expired = pk->expiredate> _cdk_timestamp ()?0 :1;
+                    } 
+                  if (key_usage && !pk->pubkey_usage)
+                    pk->pubkey_usage = key_usage;
+                }
+            }
+        }
     }
-
   return 0;
 }
 
 
-static void
-keydb_set_expire (cdkPKT_public_key * pk, u32 pk_expired)
-{
-  u32 curtime = 0;
-
-  if (pk)
-    {
-      curtime = _cdk_timestamp ();
-      if (pk->version == 3)
-          pk->expiredate = (pk->expiredate * 86400) + pk->timestamp;          
-      else
-          pk->expiredate = pk->timestamp + pk_expired;
-      pk->has_expired = pk->expiredate >= curtime? 1 : 0;
-    }
-}
-
-
 static int
-keydb_parse_allsigs (CDK_KBNODE key, int check)
+keydb_parse_allsigs (CDK_KBNODE knode, int check)
 {
   CDK_KBNODE k, kb;
-  cdkPKT_signature *sig;
-  struct cdk_subpkt_s *s = NULL;
-  int pk_expired = 0;
+  cdkPKT_signature * sig;
+  cdkPKT_public_key * pk;
+  struct cdk_subpkt_s * s = NULL;
+  u32 expiredate = 0, curtime = _cdk_timestamp ();
   int rc = 0;
 
-  if (!key)
+  if (!knode)
     return CDK_Inv_Value;
 
-  kb = cdk_kbnode_find (key, CDK_PKT_SECRET_KEY);
+  kb = cdk_kbnode_find (knode, CDK_PKT_SECRET_KEY);
   if (kb)
     return 0;
 
   /* reset */
-  for (k = key; k; k = k->next)
+  for (k = knode; k; k = k->next)
     {
       if (k->pkt->pkttype == CDK_PKT_USER_ID)
 	k->pkt->pkt.user_id->is_revoked = 0;
@@ -920,7 +928,7 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	k->pkt->pkt.public_key->is_revoked = 0;
     }
 
-  for (k = key; k; k = k->next)
+  for (k = knode; k; k = k->next)
     {
       if (k->pkt->pkttype == CDK_PKT_SIGNATURE)
 	{
@@ -928,7 +936,7 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	  /* Revocation certificates for primary keys */
 	  if (sig->sig_class == 0x20)
 	    {
-	      kb = cdk_kbnode_find_prev (key, k, CDK_PKT_PUBLIC_KEY);
+	      kb = cdk_kbnode_find_prev (knode, k, CDK_PKT_PUBLIC_KEY);
 	      if (kb)
 		{
 		  kb->pkt->pkt.public_key->is_revoked = 1;
@@ -941,7 +949,7 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	  /* Revocation certificates for subkeys */
 	  else if (sig->sig_class == 0x28)
 	    {
-	      kb = cdk_kbnode_find_prev (key, k, CDK_PKT_PUBLIC_SUBKEY);
+	      kb = cdk_kbnode_find_prev (knode, k, CDK_PKT_PUBLIC_SUBKEY);
 	      if (kb)
 		{
 		  kb->pkt->pkt.public_key->is_revoked = 1;
@@ -954,7 +962,7 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	  /* Revocation certifcates for user ID's */
 	  else if (sig->sig_class == 0x30)
 	    {
-	      kb = cdk_kbnode_find_prev (key, k, CDK_PKT_USER_ID);
+	      kb = cdk_kbnode_find_prev (knode, k, CDK_PKT_USER_ID);
 	      if (kb)
 		{
 		  kb->pkt->pkt.user_id->is_revoked = 1;
@@ -967,16 +975,18 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	  /* Direct certificates for primary keys */
 	  else if (sig->sig_class == 0x1F)
 	    {
-	      kb = cdk_kbnode_find_prev (key, k, CDK_PKT_PUBLIC_KEY);
+	      kb = cdk_kbnode_find_prev (knode, k, CDK_PKT_PUBLIC_KEY);
 	      if (kb)
 		{
-		  kb->pkt->pkt.public_key->is_invalid = 0;
+                  pk = kb->pkt->pkt.public_key;
+		  pk->is_invalid = 0;
 		  s = cdk_subpkt_find (k->pkt->pkt.signature->hashed,
 				       SIGSUBPKT_KEY_EXPIRE);
 		  if (s)
 		    {
-		      pk_expired = buffer_to_u32 (s->d);
-		      keydb_set_expire (kb->pkt->pkt.public_key, pk_expired);
+		      expiredate = _cdk_buftou32 (s->d);
+                      pk->expiredate = pk->timestamp + expiredate;
+                      pk->has_expired = pk->expiredate > curtime? 0 : 1;
 		    }
 		  if (check)
 		    ;
@@ -987,16 +997,18 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	  /* Direct certificates for subkeys */
 	  else if (sig->sig_class == 0x18)
 	    {
-	      kb = cdk_kbnode_find_prev (key, k, CDK_PKT_PUBLIC_SUBKEY);
+	      kb = cdk_kbnode_find_prev (knode, k, CDK_PKT_PUBLIC_SUBKEY);
 	      if (kb)
 		{
-		  kb->pkt->pkt.public_key->is_invalid = 0;
+                  pk = kb->pkt->pkt.public_key;
+		  pk->is_invalid = 0;
 		  s = cdk_subpkt_find (k->pkt->pkt.signature->hashed,
 				       SIGSUBPKT_KEY_EXPIRE);
 		  if (s)
 		    {
-		      pk_expired = buffer_to_u32 (s->d);
-		      keydb_set_expire (kb->pkt->pkt.public_key, pk_expired);
+		      expiredate = _cdk_buftou32 (s->d);
+                      pk->expiredate = pk->timestamp + expiredate;
+                      pk->has_expired = pk->expiredate > curtime? 0 : 1;
 		    }
 		  if (check)
 		    ;
@@ -1006,7 +1018,13 @@ keydb_parse_allsigs (CDK_KBNODE key, int check)
 	    }
 	}
     }
-
+  k = cdk_kbnode_find (knode, CDK_PKT_PUBLIC_KEY);
+  if (k) /* v3 public keys have no additional signatures for the key */
+    {
+      pk = k->pkt->pkt.public_key;
+      if (pk->version == 3)
+        pk->is_invalid = 0;
+    }
   return rc;
 }
 
@@ -1020,7 +1038,7 @@ cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * r_knode)
   int rc = 0, rc2 = 0, old_off;
   int key_seen = 0, got_key = 0;
 
-  if (!inp || !r_knode)
+  if (!inp)
     return CDK_Inv_Value;
 
   memset (keyid, 0, sizeof keyid);
@@ -1091,8 +1109,112 @@ cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * r_knode)
       if (!rc)
 	rc = rc2;
     }
+  if (r_knode)
+    *r_knode = got_key ? knode : NULL;
+  return rc;
+}
 
-  *r_knode = got_key ? knode : NULL;
+
+static int
+classify_data (const byte * buf, size_t len)
+{
+  int type = 0;
+  int i;
+
+  if (buf[0] == '0' && buf[1] == 'x')
+    {
+      buf += 2;
+      len -= 2;
+    }
+  
+  if (len == 8 || len == 16 || len == 40)
+    {
+      for (i = 0; i < len; i++)
+        {
+          if (!isxdigit (buf[i]))
+            break;
+        }
+      if (i == len)
+        {
+          switch (len)
+            {
+            case 8: type = CDK_DBSEARCH_SHORT_KEYID; break;
+            case 16: type = CDK_DBSEARCH_KEYID; break;
+            case 40: type = CDK_DBSEARCH_FPR; break;
+            }
+        }
+    }
+  if (!type)
+    type = CDK_DBSEARCH_SUBSTR;
+  return type;
+}
+
+
+static void
+keyid_from_str (const char * buf, u32 * keyid)
+{
+  if (!strncmp (buf, "0x", 2))
+    buf += 2;
+
+  if (strlen (buf) == 8)
+    keyid[1] = strtoul (buf, NULL, 16);
+  else if (strlen (buf) == 16)
+    {
+      keyid[0] = strtoul (buf, NULL, 16);
+      keyid[1] = strtoul (buf + 8, NULL, 16);
+    }
+}
+
+
+static void
+fpr_from_str (const char * buf, byte * fpr)
+{
+  char tmp[3];
+  int i;
+  
+  if (strlen (buf) != 40)
+    memset (fpr, 0, 20);
+  else
+    {
+      for (i = 0; i < 20; i++)
+        {
+          tmp[0] = buf[2*i];
+          tmp[1] = buf[2*i+1];
+          tmp[2] = '\0';
+          fpr[i] = strtoul (tmp, NULL, 16);
+        }
+    }
+}
+      
+
+int
+cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc, CDK_KBNODE * r_knode)
+{
+  u32 keyid[2];
+  byte fpr[20];
+  int rc;
+
+  switch (classify_data (desc, strlen (desc)))
+    {
+    case CDK_DBSEARCH_EXACT:
+    case CDK_DBSEARCH_SUBSTR:
+      rc = cdk_keydb_get_bypattern (hd, desc, r_knode);
+      break;
+      
+    case CDK_DBSEARCH_KEYID:
+    case CDK_DBSEARCH_SHORT_KEYID:
+      keyid_from_str (desc, keyid);
+      rc = cdk_keydb_get_bykeyid (hd, keyid, r_knode);
+      break;
+
+    case CDK_DBSEARCH_FPR:
+      fpr_from_str (desc, fpr);
+      rc = cdk_keydb_get_byfpr (hd, fpr, r_knode);
+      break;
+
+    default:
+      rc = CDK_Inv_Mode;
+    }
   return rc;
 }
 
@@ -1103,6 +1225,7 @@ cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret)
   CDK_KEYDB_HD hd;
   CDK_KBNODE knode, node;
   CDK_STRLIST r;
+  int old_ctb = 0;
   int rc = 0;
 
   hd = cdk_keydb_get_ctx (secret, -1);
@@ -1111,7 +1234,7 @@ cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret)
 
   for (r = remusr; r; r = r->next)
     {
-      rc = cdk_keydb_get_bypattern (hd, r->d, &knode);
+      rc = cdk_keydb_search_auto (hd, r->d, &knode);
       if (rc)
 	break;
       for (node = knode; node; node = node->next)
@@ -1119,11 +1242,14 @@ cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret)
 	  /* those packets are not intended for the real wolrd */
 	  if (node->pkt->pkttype == CDK_PKT_RING_TRUST)
 	    continue;
+          if (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
+              && node->pkt->pkt.public_key->version == 3)
+            old_ctb = 1;
 	  /* we never export local signed signatures */
 	  if (node->pkt->pkttype == CDK_PKT_SIGNATURE &&
 	      !node->pkt->pkt.signature->flags.exportable)
             continue;
-
+          node->pkt->old_ctb = old_ctb;
 	  rc = cdk_pkt_build (out, node->pkt);
 	  if (rc)
 	    break;
