@@ -49,6 +49,11 @@ static int log_level = CDK_LOG_NONE;
 static CDK_PASSPHRASE_CB passphrase_cb = NULL;
 
 
+/**
+ * cdk_strerror: Return an error text for the given id
+ * @ec: the error number
+ *
+ **/
 const char *
 cdk_strerror (int ec)
 {
@@ -70,7 +75,7 @@ cdk_strerror (int ec)
     case CDK_MPI_Error:        return "Invalid or missformed MPI";
     case CDK_Inv_Value:        return "Invalid parameter or value";
     case CDK_Error_No_Key:     return "No key available";
-    case CDK_Chksum_Error:     return "Check for key doesn't match";
+    case CDK_Chksum_Error:     return "Check for key does not match";
     case CDK_Time_Conflict:    return "Time conflict";
     case CDK_Zlib_Error:       return "ZLIB error";
     case CDK_Weak_Key:         return "Weak key was detected";
@@ -94,6 +99,15 @@ out_of_core (size_t n)
 }
 
 
+/**
+ * cdk_set_malloc_hooks: Set private memory hooks for the lib
+ * @new_alloc_func: malloc replacement
+ * @new_alloc_secure_func: secure malloc replacement
+ * @new_realloc_func: realloc replacement
+ * @new_calloc_func: calloc replacement
+ * @new_free_func: free replacement
+ *
+ */
 void
 cdk_set_malloc_hooks (void *(*new_alloc_func) (size_t n),
 		      void *(*new_alloc_secure_func) (size_t n),
@@ -174,7 +188,7 @@ cdk_salloc (size_t size)
       _secmem_init (32768);
       secmem_init = 1;
     }
-  if (!secmem_init == 1)
+  if (secmem_init == 1)
     {
       _secmem_init (0);
       secmem_init = 2;
@@ -245,14 +259,25 @@ _cdk_logv (int level, const char *fmt, va_list arg_ptr)
 }
 
 
+/**
+ * cdk_set_log_handler: set a private handler for logging.
+ * @logfnc: the function pointer
+ * @opaque: a private values for the function
+ *
+ */
 void
-cdk_set_log_handler (CDK_LOG_FNC logfnc, void *opaque)
+cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque)
 {
   log_handler = logfnc;
   log_handler_value = opaque; 
 }
 
 
+/**
+ * cdk_set_log_level: set the verbosity level
+ * @lvl: the level
+ *
+ **/
 void
 cdk_set_log_level (int lvl)
 {
@@ -332,9 +357,25 @@ _cdk_passphrase_free (char *pw)
   free (pw);
 }
 
-  
+
 int
-cdk_handle_new (CDK_HD *r_ctx)
+_cdk_is_idea_available (void)
+{
+  int rc = 0;
+  #ifdef LIBGCRYPT_WITH_IDEA
+  rc = 1;
+  #endif
+  return rc;
+}
+
+
+/**
+ * cdk_handle_new: create a new session handle
+ * @r_ctx: context to store the handle
+ *
+ **/
+int
+cdk_handle_new (CDK_HD * r_ctx)
 {
   CDK_HD c;
   
@@ -363,6 +404,14 @@ cdk_handle_new (CDK_HD *r_ctx)
 }
 
 
+/**
+ * cdk_handle_set_s2k: set a string to key mode
+ * @hd: the session handle
+ * @mode: the s2k mode
+ * @digest: the s2k digest algo
+ * @cipher: the s2k cipher algo
+ *
+ **/
 void
 cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher)
 {
@@ -375,30 +424,70 @@ cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher)
 }
 
 
+/**
+ * cdk_handle_set_cipher: set the cipher algorithm
+ * @hd: the session handle
+ * @cipher: cipher algorithm
+ *
+ * This is only used for symmetric encryption only because otherwise
+ * the algorithm is extracted from the prefs.
+ **/
 void
 cdk_handle_set_cipher (CDK_HD hd, int cipher)
 {
   if (hd)
-    hd->cipher_algo = cipher;
+    {
+      if (_cdk_cipher_test_algo (cipher))
+        cipher = GCRY_CIPHER_CAST5;
+      hd->cipher_algo = cipher;
+    }
 }
 
 
+/**
+ * cdk_handle_set_digest: set message digest algorithm
+ * @hd: the session handle
+ * @digest: the message digest algorithm
+ *
+ **/
 void
 cdk_handle_set_digest (CDK_HD hd, int digest)
 {
   if (hd)
-    hd->digest_algo = digest;
+    {
+      if (_cdk_md_test_algo (digest))
+        digest = GCRY_MD_SHA1;
+      hd->digest_algo = digest;
+    }
 }
 
 
+/**
+ * cdk_handle_set_compress: set the compression algorithm
+ * @hd: session handle
+ * @algo: compression algo
+ *
+ **/
 void
 cdk_handle_set_compress (CDK_HD hd, int algo)
 {
   if (hd)
-    hd->compress_algo = algo;
+    {
+      if (algo < 0 || algo > 2)
+        algo = 0;
+      hd->compress_algo = algo;
+      if (!algo)
+        hd->opt.compress = 0;
+    }
 }
 
 
+/**
+ * cdk_handle_set_armor: en- or disable ascii armor
+ * @hd: session handle
+ * @val: boolean value (1=yes, 0=no)
+ *
+ **/
 void
 cdk_handle_set_armor (CDK_HD hd, int val)
 {
@@ -407,6 +496,14 @@ cdk_handle_set_armor (CDK_HD hd, int val)
 }
 
 
+/**
+ * cdk_handle_set_compat: set compatibility mode for PGP.
+ * @hd: session handle
+ * @val: boolean value (1=yes, 0=no, -1=pgp2)
+ *
+ * If you enable this mode, only algorithms are used which are part
+ * of older PGP versions.
+ **/
 void
 cdk_handle_set_compat (CDK_HD hd, int val)
 {
@@ -416,9 +513,12 @@ cdk_handle_set_compat (CDK_HD hd, int val)
       if (!val)
         return;
       hd->opt.mdc = 0;
+      hd->opt.rfc1991 = val == -1? 1: 0;
       hd->compress_algo = CDK_COMPRESS_ZIP;
-      hd->cipher_algo = GCRY_CIPHER_CAST5;
-      hd->digest_algo = GCRY_MD_SHA1;
+      hd->cipher_algo = val == -1? GCRY_CIPHER_IDEA : GCRY_CIPHER_CAST5;
+      hd->digest_algo = val == -1? GCRY_MD_MD5: GCRY_MD_SHA1;
+      if (val == -1)
+        cdk_handle_set_s2k (hd, 0, hd->digest_algo, hd->cipher_algo);
     }
 }
 
