@@ -50,23 +50,27 @@ do \
     } \
 } while (0)
 
+/*-- armor.c --*/
+const char * _cdk_armor_get_lineend (void);
+     
 /*-- main.c --*/
 int _cdk_get_log_level (void);
 void _cdk_log_info (const char * fmt, ...);
 void _cdk_log_debug (const char * fmt, ...);
 char * _cdk_passphrase_get (const char * prompt);
 void _cdk_passphrase_free (char * pw);
+int _cdk_is_idea_available (void);
 
 /*-- misc.c --*/
 u32 _cdk_timestamp (void);
-u32 buffer_to_u32 (const byte * buf);
-void u32_to_buffer (u32 u, byte * buf);
-const char* ascii_memistr (const char * buf, size_t buflen, const char * sub);
+u32 _cdk_buftou32 (const byte * buf);
+void _cdk_u32tobuf (u32 u, byte * buf);
+const char* _cdk_memistr (const char * buf, size_t buflen, const char * sub);
 
 /*-- pubkey.c --*/
 int _cdk_pk_algo_usage (int algo);
-int _cdk_pk_algo_usage (int algo);
 int _cdk_pk_test_algo (int algo, unsigned int usage);
+int _cdk_seckey_get_csum (cdkPKT_secret_key * sk);
 
 /*-- cipher.c --*/
 int _cdk_cipher_test_algo (int algo);
@@ -83,6 +87,8 @@ void _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr);
 int _cdk_signature_check (cdkPKT_public_key * pk, cdkPKT_signature *sig,
                           GCRY_MD_HD digest, int * r_expired);
 void _cdk_hash_sig_data (cdkPKT_signature * sig, GCRY_MD_HD hd);
+void _cdk_hash_userid (cdkPKT_user_id * uid, int sig_version, GCRY_MD_HD md);
+
 
 /*-- kbnode.c --*/
 void _cdk_kbnode_add (CDK_KBNODE root, CDK_KBNODE node);
@@ -109,6 +115,8 @@ CDK_KBNODE _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid);
 int _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig);
 int _cdk_sig_hash_for (int pubkey_algo, int pkt_version);
 void _cdk_trim_string (char * s, int canon);
+int _cdk_sig_complete (cdkPKT_signature * sig, cdkPKT_secret_key * sk,
+                       GCRY_MD_HD hd);     
 
 /*-- stream.c --*/
 void * _cdk_stream_get_opaque (CDK_STREAM s, int id);
@@ -116,6 +124,7 @@ const char * _cdk_stream_get_fname (CDK_STREAM s);
 FILE * _cdk_stream_get_fp (CDK_STREAM s);
 int _cdk_stream_gets (CDK_STREAM s, char * buf, size_t count);
 int _cdk_stream_append (const char * file, CDK_STREAM * ret_s);
+int _cdk_stream_get_errno (CDK_STREAM s);
 
 /*-- verify.c --*/
 void _cdk_result_verify_free (Verify_Result res);
