@@ -34,12 +34,12 @@
 u32
 _cdk_timestamp (void)
 {
-    return (u32) time (NULL);
+  return (u32)time (NULL);
 }
 
 
 u32
-buffer_to_u32 (const byte *buf)
+_cdk_buftou32 (const byte *buf)
 {
   u32 u = 0;
   
@@ -55,7 +55,7 @@ buffer_to_u32 (const byte *buf)
 
 
 void
-u32_to_buffer (u32 u, byte *buf)
+_cdk_u32tobuf (u32 u, byte *buf)
 {
   if (buf)
     {
@@ -195,7 +195,7 @@ cdk_strlist_walk (CDK_STRLIST root, CDK_STRLIST * context)
 
 
 const char *
-ascii_memistr (const char *buf, size_t buflen, const char *sub)
+_cdk_memistr (const char *buf, size_t buflen, const char *sub)
 {
   const byte *t, *s;
   size_t n;
