@@ -158,7 +158,9 @@ _cdk_free_seckey (cdkPKT_secret_key * sk)
       nskey = cdk_pk_get_nskey (sk->pubkey_algo);
       while (sk->mpi && nskey--)
         {
+          #ifndef __MINGW32__ /* in some cases, it caused a segv */
           wipemem (sk->mpi[nskey], sk->mpi[nskey]->bytes);
+          #endif
           cdk_free (sk->mpi[nskey]);
           sk->mpi[nskey] = NULL; 
         }
@@ -167,6 +169,7 @@ _cdk_free_seckey (cdkPKT_secret_key * sk)
       _cdk_free_pubkey (sk->pk);
       sk->pk = NULL;
       cdk_free (sk->protect.s2k);
+      sk->protect.s2k = NULL;
       cdk_free (sk);
     }
 }
@@ -314,7 +317,7 @@ _cdk_copy_prefs (const struct cdk_prefitem_s * prefs)
       new_prefs[n].type = prefs[n].type;
       new_prefs[n].value = prefs[n].value;
     }
-  new_prefs[n].type = PREFTYPE_NONE;
+  new_prefs[n].type = CDK_PREFTYPE_NONE;
   new_prefs[n].value = 0;
 
   return new_prefs;
@@ -510,10 +513,10 @@ cdk_subpkt_free (CDK_SUBPKT ctx)
 }
 
 
-struct cdk_subpkt_s *
+CDK_SUBPKT
 cdk_subpkt_find (CDK_SUBPKT ctx, int type)
 {
-  struct cdk_subpkt_s *s;
+  CDK_SUBPKT s;
 
   for (s = ctx; s; s = s->next)
     {
@@ -533,7 +536,6 @@ cdk_subpkt_new (size_t size)
   s = cdk_calloc (1, sizeof *s + size + 1);
   if (!s)
     return NULL;
-
   return s;
 }
 
@@ -567,14 +569,18 @@ _cdk_subpkt_add (CDK_SUBPKT root, CDK_SUBPKT node)
 
 
 byte *
-_cdk_subpkt_get_array (CDK_SUBPKT s, int count, size_t *r_nbytes)
+_cdk_subpkt_get_array (CDK_SUBPKT s, int count, size_t * r_nbytes)
 {
   CDK_SUBPKT list;
-  byte *buf;
+  byte * buf;
   int n, nbytes;
   
   if (!s)
-    return NULL;
+    {
+      if (r_nbytes)
+        *r_nbytes = 0;
+      return NULL;
+    }
 
   buf = cdk_calloc (1, 4096);
   if (!buf)
@@ -638,15 +644,14 @@ _cdk_subpkt_copy (struct cdk_subpkt_s ** r_dst, struct cdk_subpkt_s * src)
         _cdk_subpkt_add (root, node);
     }
   *r_dst = root;
-
   return 0;
 }
 
 
 int
-_cdk_subpkt_hash (CDK_SUBPKT hashed, size_t *r_nbytes, GCRY_MD_HD hd)
+_cdk_subpkt_hash (CDK_SUBPKT hashed, size_t * r_nbytes, GCRY_MD_HD hd)
 {
-  byte *p, buf[2];
+  byte * p, buf[2];
   size_t nbytes;
 
   p = _cdk_subpkt_get_array (hashed, 0, &nbytes);
@@ -660,7 +665,6 @@ _cdk_subpkt_hash (CDK_SUBPKT hashed, size_t *r_nbytes, GCRY_MD_HD hd)
   gcry_md_write (hd, p, nbytes);
   if (r_nbytes)
     *r_nbytes = nbytes;
-  
   return 0;
 }
 
