@@ -21,7 +21,7 @@
 #ifndef OPENCDK_H
 #define OPENCDK_H
 
-#define OPENCDK_VERSION "0.3.3"
+#define OPENCDK_VERSION "0.3.4"
 
 #include <gcrypt.h>
 
@@ -62,6 +62,9 @@ typedef struct cdk_key_list_s *CDK_KEYLIST;
 struct cdk_subpkt_s;
 typedef struct cdk_subpkt_s *CDK_SUBPKT;
 
+struct cdk_keygen_s;
+typedef struct cdk_keygen_s *CDK_KEYGEN_CTX;
+
 
 typedef enum
 {
@@ -109,14 +112,22 @@ enum
 
 enum
 {
-  CDK_PK_RSA = 1,
-  CDK_PK_RSA_E = 2,
-  CDK_PK_RSA_S = 3,
+  CDK_PK_RSA   =  1,
+  CDK_PK_RSA_E =  2,
+  CDK_PK_RSA_S =  3,
   CDK_PK_ELG_E = 16,
   CDK_PK_DSA   = 17,
   CDK_PK_ELG   = 20
 };
 
+enum
+{
+  CDK_PREFTYPE_NONE = 0,
+  CDK_PREFTYPE_SYM  = 1,
+  CDK_PREFTYPE_HASH = 2,
+  CDK_PREFTYPE_ZIP  = 3
+};
+
 enum
 {
   CDK_ARMOR_MESSAGE   = 0,
@@ -194,25 +205,25 @@ enum
 
 typedef enum
 {
-  CDK_PKT_RESERVED = 0,
-  CDK_PKT_PUBKEY_ENC = 1,
-  CDK_PKT_SIGNATURE = 2,
-  CDK_PKT_SYMKEY_ENC = 3,
-  CDK_PKT_ONEPASS_SIG = 4,
-  CDK_PKT_SECRET_KEY = 5,
-  CDK_PKT_PUBLIC_KEY = 6,
-  CDK_PKT_SECRET_SUBKEY = 7,
-  CDK_PKT_COMPRESSED = 8,
-  CDK_PKT_ENCRYPTED = 9,
-  CDK_PKT_MARKER = 10,
-  CDK_PKT_PLAINTEXT = 11,
-  CDK_PKT_RING_TRUST = 12,
-  CDK_PKT_USER_ID = 13,
+  CDK_PKT_RESERVED      =  0,
+  CDK_PKT_PUBKEY_ENC    =  1,
+  CDK_PKT_SIGNATURE     =  2,
+  CDK_PKT_SYMKEY_ENC    =  3,
+  CDK_PKT_ONEPASS_SIG   =  4,
+  CDK_PKT_SECRET_KEY    =  5,
+  CDK_PKT_PUBLIC_KEY    =  6,
+  CDK_PKT_SECRET_SUBKEY =  7,
+  CDK_PKT_COMPRESSED    =  8,
+  CDK_PKT_ENCRYPTED     =  9,
+  CDK_PKT_MARKER        = 10,
+  CDK_PKT_PLAINTEXT     = 11,
+  CDK_PKT_RING_TRUST    = 12,
+  CDK_PKT_USER_ID       = 13,
   CDK_PKT_PUBLIC_SUBKEY = 14,
-  CDK_PKT_OLD_COMMENT = 16,
-  CDK_PKT_ATTRIBUTE = 17,
+  CDK_PKT_OLD_COMMENT   = 16,
+  CDK_PKT_ATTRIBUTE     = 17,
   CDK_PKT_ENCRYPTED_MDC = 18,
-  CDK_PKT_MDC = 19,
+  CDK_PKT_MDC           = 19,
 } cdk_packet_types_t;
 
 
@@ -223,7 +234,7 @@ typedef struct
   unsigned is_revoked:1;
   unsigned mdc_feature:1;
   CDK_PREFITEM prefs;
-  unsigned char * attrib_img; /* TAG 17 if not null */
+  unsigned char * attrib_img; /* Tag 17 if not null */
   size_t attrib_len;
   size_t prefs_size;
   unsigned long created;
@@ -238,7 +249,7 @@ typedef struct
   unsigned long keyid[2];
   unsigned long main_keyid[2];
   unsigned long timestamp;
-  unsigned short expiredate;
+  unsigned long expiredate;
   CDK_MPI mpi[4];
   unsigned is_revoked:1;
   unsigned is_invalid:1;
@@ -253,7 +264,7 @@ typedef struct
 typedef struct
 {
   cdkPKT_public_key * pk;
-  unsigned short expiredate;
+  unsigned long expiredate;
   int version;
   int pubkey_algo;
   unsigned long keyid[2];
@@ -304,8 +315,7 @@ typedef struct
     unsigned checked:1;
     unsigned valid:1;
     unsigned missing_key:1;
-  } flags;
-  
+  } flags;  
   unsigned long key[2]; /* only valid for key signatures */
 } cdkPKT_signature;
 
@@ -379,6 +389,7 @@ struct cdk_packet_s
 {
   int pktlen; /* real packet length */
   int pktsize; /* length with all headers */
+  int old_ctb;
   cdk_packet_types_t pkttype;
   union
   {
@@ -429,6 +440,10 @@ void cdk_handle_set_callback (CDK_HD hd,
                               void (*cb) (void *opa, int type, const char * s),
                               void * cb_value);
 
+/*-- cipher.c --*/
+void cdk_set_progress_handler (void (*cb)(void * hd, unsigned off,
+                                          unsigned size), void * cb_value);
+
 /*-- new-packet.c --*/
 int cdk_pkt_new (CDK_PACKET ** r_pkt);
 int cdk_pkt_alloc (CDK_PACKET ** r_pkt, int id);
@@ -495,7 +510,7 @@ int cdk_stream_control (CDK_STREAM s, int ctl, int val);
 int cdk_stream_open (const char * file, CDK_STREAM * ret_s);
 int cdk_stream_new (const char * file, CDK_STREAM * ret_s);
 CDK_STREAM cdk_stream_tmp (void);
-CDK_STREAM cdk_stream_tmp_from_mem (const unsigned char * buf, size_t count);
+CDK_STREAM cdk_stream_tmp_from_mem (const void * buf, size_t count);
 void cdk_stream_tmp_set_mode (CDK_STREAM s, int val);
 CDK_STREAM cdk_stream_fpopen (FILE * fp, unsigned write_mode);
 int cdk_stream_flush (CDK_STREAM s);
@@ -519,8 +534,7 @@ int cdk_stream_set_text_flag (CDK_STREAM s, const char * lf);
 int cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out);
 
 /*-- keydb.c --*/
-int cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type,
-                   unsigned char * data, size_t count);
+int cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count);
 int cdk_keydb_open (CDK_KEYDB_HD kdb, CDK_STREAM * ret_kr);
 int cdk_keydb_add_resource (const char * name, int is_secret);
 int cdk_keydb_remove_resource (int id);
@@ -531,6 +545,8 @@ CDK_KEYDB_HD cdk_keydb_get_ctx (int is_secret, int id);
 int cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc);
 void cdk_keydb_search_free (CDK_DBSEARCH dbs);
 int cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key);
+int cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc,
+                           CDK_KBNODE * r_knode);
 void cdk_keydb_search_reset (CDK_KEYDB_HD hd);
 int cdk_keydb_get_bykeyid (CDK_KEYDB_HD kdb, unsigned long * keyid,
                            CDK_KBNODE * r_key);
@@ -632,5 +648,17 @@ char * cdk_utf8_decode (const char * string, size_t length, int delim);
 int cdk_keyserver_recv_key (const char * host, int port,
                             unsigned long * keyid, CDK_KBNODE * r_knode);
 
+/*-- keygen.c --*/
+int cdk_keygen_set_prefs (CDK_KEYGEN_CTX hd, int type, unsigned char * array,
+                          size_t n);
+int cdk_keygen_set_algo_info (CDK_KEYGEN_CTX hd, int type, int algo, int bits);
+void cdk_keygen_set_mdc_feature (CDK_KEYGEN_CTX hd, int val);
+void cdk_keygen_set_expire_date (CDK_KEYGEN_CTX hd, long timestamp);
+void cdk_keygen_set_name (CDK_KEYGEN_CTX hd, const char * name);
+int cdk_keygen_start (CDK_KEYGEN_CTX hd);
+int cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec);
+int cdk_keygen_new (CDK_KEYGEN_CTX * r_hd);
+void cdk_keygen_free (CDK_KEYGEN_CTX hd);
+
 #endif /* OPENCDK_H */
 
