@@ -47,14 +47,6 @@ enum
   SIGSUBPKT_FEATURES = 30
 };
 
-enum
-{
-  PREFTYPE_NONE = 0,
-  PREFTYPE_SYM = 1,
-  PREFTYPE_HASH = 2,
-  PREFTYPE_ZIP = 3
-};
-
 
 struct cdk_kbnode_s
 {
