@@ -39,13 +39,15 @@ plaintext_decode (void * opaque, FILE * in, FILE * out)
   size_t nread;
   int rc, bufsize;
 
+  _cdk_log_debug ("plaintext filter: decode\n");
+  
   if (!pfx || !in || !out)
     return CDK_Inv_Value;
 
-  si = cdk_stream_fpopen (in, 0);
+  si = cdk_stream_fpopen (in, S_READ);
   if (!si)
     return CDK_Out_Of_Core;
-  so = cdk_stream_fpopen (out, 1);
+  so = cdk_stream_fpopen (out, S_WRITE);
   if (!so)
     {
       cdk_stream_close (si);
@@ -98,16 +100,18 @@ plaintext_encode (void * opaque, FILE * in, FILE * out)
   size_t filelen;
   int rc;
 
+  _cdk_log_debug ("plaintext filter: encode\n");
+  
   if (!pfx || !in || !out)
     return CDK_Inv_Value;
   
   if (!pfx->filename)
     pfx->filename = cdk_strdup ("_CONSOLE");
 
-  si = cdk_stream_fpopen (in, 0);
+  si = cdk_stream_fpopen (in, S_READ);
   if (!si)
     return CDK_Out_Of_Core;
-  so = cdk_stream_fpopen (out, 1);
+  so = cdk_stream_fpopen (out, S_WRITE);
   if (!so)
     {
       cdk_stream_close (si);
@@ -125,6 +129,7 @@ plaintext_encode (void * opaque, FILE * in, FILE * out)
   pt->len = cdk_stream_get_length (si);
   pt->buf = si;
   init_packet (&pkt);
+  pkt.old_ctb = pfx->rfc1991? 1 : 0;
   pkt.pkttype = CDK_PKT_PLAINTEXT;
   pkt.pkt.plaintext = pt;
   rc = cdk_pkt_build (so, &pkt);
@@ -143,6 +148,9 @@ text_encode (void * opaque, FILE * in, FILE * out)
   const char * s;
   char buf[1024];
 
+  if (!in || !out)
+    return CDK_Inv_Value;
+  
   while (!feof (in))
     {
       s = fgets (buf, sizeof buf-1, in);
@@ -159,11 +167,11 @@ text_encode (void * opaque, FILE * in, FILE * out)
 int
 text_decode (void * opaque, FILE * in, FILE * out)
 {
+  text_filter_s * tfx = opaque;
   const char * s;
   char buf[1024];
-  text_filter_s * tfx = opaque;
 
-  if (!tfx)
+  if (!tfx || !in || !out)
     return CDK_Inv_Value;
 
   while (!feof (in))
