@@ -37,7 +37,7 @@ static int sexp_to_enc (cdkPKT_pubkey_enc * enc, GCRY_SEXP sexp);
 static int enc_to_sexp (GCRY_SEXP * r_sexp, cdkPKT_pubkey_enc * enc);
 
 
-#if 1
+#ifdef DEBUG
 void
 dump_sexp (GCRY_SEXP sexp)
 {
@@ -641,7 +641,6 @@ cdk_pk_get_mpi (cdkPKT_public_key * pk, int idx,
 {
   if (!pk || idx < 0 || !r_count)
     return CDK_Inv_Value;
-
   if (idx > cdk_pk_get_npkey (pk->pubkey_algo))
     return CDK_Inv_Value;
   return read_mpi (pk->mpi[idx], buf, r_count, r_nbits);
@@ -793,3 +792,18 @@ cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw)
 
   return 0;
 }
+
+
+int
+_cdk_seckey_get_csum (cdkPKT_secret_key * sk)
+{
+  u16 csum = 0, i;
+
+  for (i = 0; i < cdk_pk_get_nskey (sk->pubkey_algo); i++)
+    csum += checksum_mpi (sk->mpi[i]);
+  return csum;
+}
+
+  
+  
+
