@@ -396,7 +396,7 @@ read_compressed (CDK_STREAM inp, size_t pktlen, cdkPKT_compressed * c)
 static int
 read_public_key (CDK_STREAM inp, cdkPKT_public_key * pk)
 {
-  int i = 0;
+  int i = 0, ndays;
   int rc = 0;
 
   if (!inp || !pk)
@@ -414,7 +414,11 @@ read_public_key (CDK_STREAM inp, cdkPKT_public_key * pk)
       return CDK_Inv_Packet_Ver;
   pk->timestamp = read_32 (inp, &rc);
   if (pk->version < 4)
-    pk->expiredate = read_16 (inp, &rc);
+    {
+      ndays = read_16 (inp, &rc);
+      if (ndays)
+        pk->expiredate = pk->timestamp + ndays * 86400L;
+    }
   pk->pubkey_algo = stream_getc (inp);
   for (i = 0; i < cdk_pk_get_npkey (pk->pubkey_algo); i++)
     {
@@ -571,7 +575,7 @@ read_attribute (CDK_STREAM inp, size_t pktlen, cdkPKT_user_id * attr)
   len = *p++;
   if (len == 255)
     {
-      len = buffer_to_u32 (p);
+      len = _cdk_buftou32 (p);
       p += 4;
       pktlen -= 4;
     }
@@ -619,7 +623,6 @@ read_user_id (CDK_STREAM inp, size_t pktlen, cdkPKT_user_id * user_id)
   rc = stream_read (inp, user_id->name, pktlen, &nread);
   if (!rc && nread != pktlen)
     return CDK_Inv_Packet;
-
   user_id->name[nread] = '\0';
   return rc;
 }
@@ -714,7 +717,7 @@ read_onepass_sig (CDK_STREAM inp, size_t pktlen, cdkPKT_onepass_sig * sig)
 static int
 read_signature (CDK_STREAM inp, size_t pktlen, cdkPKT_signature * sig)
 {
-  struct cdk_subpkt_s *p = NULL;
+  struct cdk_subpkt_s * node = NULL;
   size_t nbytes;
   int i, size;
   int rc = 0;
@@ -783,36 +786,36 @@ read_signature (CDK_STREAM inp, size_t pktlen, cdkPKT_signature * sig)
 
       /* Setup the standard packet entries, so we can use V4
          signatures similar to V3. */
-      for (p = sig->unhashed; p; p = p->next)
+      for (node = sig->unhashed; node; node = node->next)
 	{
-	  if (p->type == SIGSUBPKT_ISSUER)
+	  if (node->type == SIGSUBPKT_ISSUER)
 	    {
-	      sig->keyid[0] = buffer_to_u32 (p->d);
-	      sig->keyid[1] = buffer_to_u32 (p->d + 4);
+	      sig->keyid[0] = _cdk_buftou32 (node->d);
+	      sig->keyid[1] = _cdk_buftou32 (node->d + 4);
 	    }
-	  else if (p->type == SIGSUBPKT_EXPORTABLE && p->d[0] == 0)
+	  else if (node->type == SIGSUBPKT_EXPORTABLE && node->d[0] == 0)
 	    {
 	      /* this packet might be also placed in the unhashed area */
 	      sig->flags.exportable = 0;
 	    }
 	}
-      for (p = sig->hashed; p; p = p->next)
+      for (node = sig->hashed; node; node = node->next)
 	{
-	  if (p->type == SIGSUBPKT_SIG_CREATED)
-	    sig->timestamp = buffer_to_u32 (p->d);
-	  else if (p->type == SIGSUBPKT_SIG_EXPIRE)
+	  if (node->type == SIGSUBPKT_SIG_CREATED)
+	    sig->timestamp = _cdk_buftou32 (node->d);
+	  else if (node->type == SIGSUBPKT_SIG_EXPIRE)
 	    {
-	      sig->expiredate = buffer_to_u32 (p->d);
+	      sig->expiredate = _cdk_buftou32 (node->d);
 	      if (sig->expiredate > 0 && sig->expiredate < _cdk_timestamp ())
 		sig->flags.expired = 1;
 	    }
-	  else if (p->type == SIGSUBPKT_POLICY)
+	  else if (node->type == SIGSUBPKT_POLICY)
 	    sig->flags.policy_url = 1;
-	  else if (p->type == SIGSUBPKT_NOTATION)
+	  else if (node->type == SIGSUBPKT_NOTATION)
 	    sig->flags.notation = 1;
-	  else if (p->type == SIGSUBPKT_REVOCABLE && p->d[0] == 0)
+	  else if (node->type == SIGSUBPKT_REVOCABLE && node->d[0] == 0)
 	    sig->flags.revocable = 0;
-	  else if (p->type == SIGSUBPKT_EXPORTABLE && p->d[0] == 0)
+	  else if (node->type == SIGSUBPKT_EXPORTABLE && node->d[0] == 0)
 	    sig->flags.exportable = 0;
 	}
       sig->digest_start[0] = stream_getc (inp);
@@ -885,7 +888,7 @@ read_old_length (PKTHEAD * ph, CDK_STREAM inp)
     {
       ph->len = 0;
       ph->size = 0;
-    }  
+    }
 }
 
 
@@ -917,6 +920,14 @@ read_new_length (PKTHEAD * ph, CDK_STREAM inp)
 }
 
 
+static void
+skip_packet (CDK_STREAM inp, int pktsize)
+{
+  while (pktsize--)
+    stream_getc (inp);  
+}
+
+
 int
 cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt)
 {
@@ -965,6 +976,7 @@ cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt)
   pkt->pkttype = pkth.type;
   pkt->pktlen = pkth.len;
   pkt->pktsize = pkth.size + pkth.len;
+  pkt->old_ctb = pkth.rfc2440? 0 : 1;
 
   switch (pkt->pkttype)
     {
@@ -1080,8 +1092,7 @@ cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt)
 
     default:
       /* skip all packets we don't understand */
-      while (pkth.len--)
-        stream_getc (inp);
+      skip_packet (inp, pkth.len);
       break;
     }
 
