@@ -30,7 +30,7 @@
 #include "packet.h"
 
 #define gcry_md_get_asnoid(algo, asnbuf, asnlen) \
-gcry_md_algo_info( (algo), GCRYCTL_GET_ASNOID, (asnbuf), (asnlen) )
+gcry_md_algo_info ((algo), GCRYCTL_GET_ASNOID, (asnbuf), (asnlen))
 
 /* We encode the MD in this way:
  *
@@ -79,7 +79,6 @@ do_encode_md (byte ** r_frame, size_t * r_flen, const byte * md, int algo,
     *r_frame = frame;
   if (r_flen)
     *r_flen = n;
-
   return 0;
 }
 
@@ -96,13 +95,12 @@ do_encode_md (byte ** r_frame, size_t * r_flen, const byte * md, int algo,
 int
 _cdk_pkcs1_sesskey (GCRY_MPI * esk, CDK_DEK dek, int nbits)
 {
-  int rc = 0;
-  int i = 0;
-  int nframe;
-  byte *p, *frame;
-  u16 chksum = 0;
-  size_t n = 0;
   GCRY_MPI a = NULL;
+  byte * p, * frame;
+  size_t n = 0;
+  u16 chksum = 0;
+  int i = 0, nframe = 0;
+  int rc = 0;
 
   if (!esk || !dek)
     return CDK_Inv_Value;
@@ -122,7 +120,7 @@ _cdk_pkcs1_sesskey (GCRY_MPI * esk, CDK_DEK dek, int nbits)
   for (;;)
     {
       int j, k;
-      byte *pp;
+      byte * pp;
 
       /* count the zero bytes */
       for (j = k = 0; j < i; j++)
@@ -249,7 +247,7 @@ _cdk_pkcs1_digest (byte ** r_md, size_t * r_mdlen, int pk_algo,
     }
   else
     {
-      byte *asn = NULL;
+      byte * asn = NULL;
       size_t asnlen = 0, digest_size = 0;
 
       rc = gcry_md_get_asnoid (digest_algo, NULL, &asnlen);
@@ -267,7 +265,6 @@ _cdk_pkcs1_digest (byte ** r_md, size_t * r_mdlen, int pk_algo,
       cdk_free (asn);
       return rc;
     }
-
   return 0;
 }
 
@@ -392,7 +389,7 @@ cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD s2k, int mode, char * pw)
     dek->keylen = 0;
   else
     cdk_hash_passphrase (dek, pw, s2k, mode == 2);
-  return dek;    
+  return dek;
 }
 
 
