@@ -95,8 +95,8 @@ _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr)
 }
 
 
-static void
-hash_userid (cdkPKT_user_id * uid, int sig_version, GCRY_MD_HD md)
+void
+_cdk_hash_userid (cdkPKT_user_id * uid, int sig_version, GCRY_MD_HD md)
 {
   const byte *data;
   byte buf[5];
@@ -298,7 +298,7 @@ check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
       if (!node)
 	return CDK_Error_No_Key; /* no user ID for key signature packet */
       _cdk_hash_pubkey (pk, md, 0);
-      hash_userid (node->pkt->pkt.user_id, sig->version, md);
+      _cdk_hash_userid (node->pkt->pkt.user_id, sig->version, md);
       if (pk->keyid[0] == sig->keyid[0] && pk->keyid[1] == sig->keyid[1])
 	rc = _cdk_signature_check (pk, sig, md, &is_expired);
       else if (hd)
