@@ -32,6 +32,27 @@
 #include "stream.h"
 
 
+static void
+sig_calc_subpkt_size (cdkPKT_signature * sig)
+{
+  size_t nbytes;
+  byte * p;
+  
+  if (sig->hashed)
+    {
+      p = _cdk_subpkt_get_array (sig->hashed, 1, &nbytes);
+      sig->hashed_size = nbytes;
+      cdk_free (p);
+    }
+  if (sig->unhashed)
+    {
+      p = _cdk_subpkt_get_array (sig->unhashed, 1, &nbytes);
+      sig->unhashed_size = nbytes;
+      cdk_free (p);
+    }  
+}
+
+  
 int
 _cdk_sig_hash_for (int pubkey_algo, int pktver)
 {
@@ -46,9 +67,8 @@ _cdk_sig_hash_for (int pubkey_algo, int pktver)
 int
 _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig)
 {
+  CDK_SUBPKT node;
   byte buf[8];
-  size_t nbytes;
-  struct cdk_subpkt_s * node;
 
   if (!sig)
     return CDK_Inv_Value;
@@ -67,31 +87,31 @@ _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig)
   
   sig->hashed = sig->unhashed = NULL;
 
-  u32_to_buffer (sig->keyid[0], buf);
-  u32_to_buffer (sig->keyid[1], buf + 4);
+  _cdk_u32tobuf (sig->keyid[0], buf);
+  _cdk_u32tobuf (sig->keyid[1], buf + 4);
   node = cdk_subpkt_new (8);
   if (node)
     {
       node->type = SIGSUBPKT_ISSUER;
-      memcpy (node->d, buf, 8);
       node->size = 8;
+      memcpy (node->d, buf, 8);
     }
   sig->unhashed = node;
   
-  u32_to_buffer (sig->timestamp, buf);
+  _cdk_u32tobuf (sig->timestamp, buf);
   node = cdk_subpkt_new (4);
   if (node)
     {
       node->type = SIGSUBPKT_SIG_CREATED;
-      memcpy (node->d, buf, 4);
       node->size = 4;
+      memcpy (node->d, buf, 4);
     }
   sig->hashed = node;
   
   if (sig->expiredate)
     {
       u32 u = sig->expiredate - sig->timestamp;
-      u32_to_buffer (u, buf);
+      _cdk_u32tobuf (u, buf);
       node = cdk_subpkt_new (4);
       if (node)
         {
@@ -101,21 +121,28 @@ _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig)
         }
       _cdk_subpkt_add (sig->hashed, node);
     }
+  sig_calc_subpkt_size (sig);
 
-  if (sig->hashed)
-    {
-      _cdk_subpkt_get_array (sig->hashed, 1, &nbytes);
-      sig->hashed_size = nbytes;
-    }
-  if (sig->unhashed)
-    {
-      _cdk_subpkt_get_array (sig->unhashed, 1, &nbytes);
-      sig->unhashed_size = nbytes;
-    }
   return 0;
 }
 
 
+int
+_cdk_sig_complete (cdkPKT_signature * sig, cdkPKT_secret_key * sk,
+                   GCRY_MD_HD md)
+{
+  byte digest[24];
+
+  sig_calc_subpkt_size (sig);
+  _cdk_hash_sig_data (sig, md);
+  gcry_md_final (md);
+
+  memcpy (digest, gcry_md_read (md, sig->digest_algo),
+          gcry_md_get_algo_dlen (sig->digest_algo));
+  return cdk_pk_sign (sk, sig, digest);
+}
+
+
 static int
 write_plaintext (CDK_STREAM inp, CDK_STREAM out)
 {
@@ -155,22 +182,34 @@ write_pubkey_enc_list (CDK_HD hd, CDK_STREAM out, CDK_STRLIST remusr)
 
   cdk_free (hd->dek);
   rc = cdk_dek_new (&hd->dek, 0);
-  if (rc)
+  if (!rc)
     {
-      cdk_pklist_release (pkl);
-      pkl = NULL;
+      hd->dek->algo = cdk_pklist_select_algo (pkl);
+      hd->dek->use_mdc = cdk_pklist_use_mdc (pkl);
+      rc = cdk_pklist_encrypt (pkl, hd->dek, out);
     }
-
-  hd->dek->algo = cdk_pklist_select_algo (pkl);
-  hd->dek->use_mdc = cdk_pklist_use_mdc (pkl);
-
-  rc = cdk_pklist_encrypt (pkl, hd->dek, out);
-  
   cdk_pklist_release (pkl);
   return rc;
 }
 
 
+static int
+get_sig_version (CDK_HD hd, CDK_KEYLIST kl)
+{
+  CDK_KEYLIST l;
+
+  if (hd->opt.compat)
+    return 3;
+  for (l = kl; l; l = l->next)
+    {
+      if ((l->type == CDK_PKT_PUBLIC_KEY && l->key.pk->version == 3)
+          || (l->type == CDK_PKT_SECRET_KEY && l->key.sk->version == 3))
+        return 3;
+    }
+  return 4;
+}
+
+
 /**
  * cdk_file_sign: Sign a file.
  * @locusr: List of userid which should be used for signing
@@ -237,7 +276,7 @@ cdk_file_sign (CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
   
   cdk_stream_set_hash_flag (inp, digest_algo);
   /* kick off the filter */
-  sig_version = hd->opt.compat? 3 : 4;
+  sig_version = get_sig_version (hd, list);
   cdk_stream_read (inp, NULL, 0);
   mfx = _cdk_stream_get_opaque (inp, fHASH);
   if (mfx && mfx->md)
@@ -262,10 +301,16 @@ leave:
 
 
 static void
-put_hash_line (CDK_STREAM out, int digest_algo)
+put_hash_line (CDK_STREAM out, int digest_algo, int key_ver)
 {
   const char * s = NULL;
-  
+
+  if (key_ver == 3)
+    {
+      cdk_stream_putc (out, '\n');
+      return;
+    }
+
   switch (digest_algo)
     {
     case GCRY_MD_MD5   : s = "Hash: MD5\n\n"; break;
@@ -342,9 +387,11 @@ cdk_file_clearsign (CDK_HD hd, CDK_STRLIST locusr,
       goto leave;
     }
 
-  s = "-----BEGIN PGP SIGNED MESSAGE-----\n";
-  cdk_stream_write (out, s, strlen (s));
-  put_hash_line (out, digest_algo);
+  s = _cdk_armor_get_lineend ();
+  strcpy (buf, "-----BEGIN PGP SIGNED MESSAGE-----");
+  strcat (buf, s);
+  cdk_stream_write (out, buf, strlen (buf));
+  put_hash_line (out, digest_algo, sk->version);
   
   while (!cdk_stream_eof (inp))
     {
@@ -358,11 +405,14 @@ cdk_file_clearsign (CDK_HD hd, CDK_STRLIST locusr,
           memmove (&buf[2], buf, nread + 1);
           buf[1] = ' ';
         }
-      buf[strlen (buf) - 1] = '\0';
-      buf[strlen (buf) - 1] = '\n';
+      if (strlen (s) == 1)
+        {
+          buf[strlen (buf) - 1] = '\0';
+          buf[strlen (buf) - 1] = '\n';
+        }
       cdk_stream_write (out, buf, strlen (buf));
     }
-  cdk_stream_putc (out, '\n');
+  cdk_stream_write (out, s, strlen (s));
   tmp = cdk_stream_tmp ();
   if (!tmp)
     {
@@ -372,7 +422,7 @@ cdk_file_clearsign (CDK_HD hd, CDK_STRLIST locusr,
   cdk_stream_tmp_set_mode (tmp, 1);
   cdk_stream_set_armor_flag (tmp, CDK_ARMOR_SIGNATURE);
 
-  sig_version = hd->opt.compat? 3 : 4;
+  sig_version = get_sig_version (hd, list);
   rc = cdk_sklist_write (list, tmp, md, 0x01, sig_version);
   if (rc)
     {
