@@ -71,6 +71,7 @@ mkstemp (char * template)
 }
 #endif
 
+
 static char *
 mk_tmpnam (void)
 {
@@ -96,6 +97,8 @@ cdk_stream_open (const char * file, CDK_STREAM * ret_s)
 
   if (!file)
     return CDK_Inv_Value;
+
+  _cdk_log_debug ("open stream `%s'\n", file);
   
   s = cdk_calloc (1, sizeof *s);
   if (!s)
@@ -190,7 +193,7 @@ cdk_stream_tmp (void)
 
 
 CDK_STREAM
-cdk_stream_tmp_from_mem (const byte * buf, size_t count)
+cdk_stream_tmp_from_mem (const void * buf, size_t count)
 {
   CDK_STREAM s;
   int nwritten;
@@ -322,9 +325,10 @@ cdk_stream_close (CDK_STREAM s)
 {
   struct stream_filter_s * f, * f2;
   int rc = 0;
-  
+
   if (s)
     {
+      _cdk_log_debug ("close stream `%s'\n", s->fname? s->fname : "[temp]");
       if (!s->flags.filtrated)
         rc = cdk_stream_flush (s);
       else if (s->fname || s->flags.temp)
@@ -378,7 +382,14 @@ _cdk_stream_get_fp (CDK_STREAM s)
 {
   return s->fp;
 }
-  
+
+
+int
+_cdk_stream_get_errno (CDK_STREAM s)
+{
+  return s->error;
+}
+
 
 /**
  * cdk_stream_get_length: Return the length of the associated file handle.
@@ -395,9 +406,15 @@ cdk_stream_get_length (CDK_STREAM s)
 
   rc = fflush (s->fp);
   if (rc)
-    return -1;
+    {
+      s->error = CDK_File_Error;
+      return -1;
+    }
   if (fstat (fileno (s->fp), &statbuf))
-    return -1;
+    {
+      s->error = CDK_File_Error;
+      return -1;
+    }
   return statbuf.st_size;
 }
 
@@ -547,6 +564,7 @@ stream_filter_walk_write (CDK_STREAM s)
             break;
         }
       rc = f->fnc (f->opaque, s->fp, f->tmp);
+      _cdk_log_debug ("filter: walk_write type=%d rc=%d\n", f->type, rc);
       if (rc)
         {
           fclose (f->tmp);
@@ -618,6 +636,7 @@ stream_filter_walk_read (CDK_STREAM s)
           break;
         }
       rc = f->fnc (f->opaque, s->fp, f->tmp);
+      _cdk_log_debug ("filter: walk_read type=%d rc=%d\n", f->type, rc);
       if (rc)
         break;
       
@@ -684,7 +703,10 @@ cdk_stream_read (CDK_STREAM s, void * buf, size_t count)
     {
       rc = stream_filter_walk_read (s);
       if (rc)
-        return EOF;
+        {
+          s->error = rc;
+          return EOF;
+        }
       s->flags.filtrated = 1;
     }
   if (!buf && !count)
@@ -707,7 +729,10 @@ cdk_stream_getc (CDK_STREAM s)
   
   nread = cdk_stream_read (s, buf, 1);
   if (nread == EOF)
+    {
+      s->error = CDK_File_Error;
       return -1;
+    }
   return buf[0];
 }
 
@@ -758,7 +783,10 @@ cdk_stream_putc (CDK_STREAM s, int c)
   buf[0] = c;
   nwritten = cdk_stream_write (s, buf, 1);
   if (nwritten == EOF)
-    return -1;
+    {
+      s->error = CDK_File_Error;
+      return -1;
+    }
   return 0;
 }
 
