@@ -34,7 +34,8 @@ enum
 
 typedef int (*FILTER_FUNC) (void *opaque, FILE * in, FILE * out);
 
-struct stream_filter_s {
+struct stream_filter_s
+{
   struct stream_filter_s * next;
   FILTER_FUNC fnc;
   void * opaque;
@@ -69,6 +70,7 @@ struct cdk_stream_context_s
 {
   struct stream_filter_s * filters;
   int fmode;
+  int error;
   struct
   {
     unsigned filtrated:1;
