@@ -44,7 +44,8 @@ stream_read (CDK_STREAM s, void * buf, size_t count, size_t * r_nread)
   int nread = cdk_stream_read (s, buf, count);
   if (nread == EOF)
     return CDK_File_Error;
-  *r_nread = nread;
+  if (r_nread)
+    *r_nread = nread;
   return 0;
 }
 
@@ -159,7 +160,7 @@ pkt_encode_len (CDK_STREAM out, size_t pktlen)
 
 
 static int
-pkt_write_head (CDK_STREAM out, size_t size, int type)
+write_head_new (CDK_STREAM out, size_t size, int type)
 {
   int rc = 0;
 
@@ -176,21 +177,67 @@ pkt_write_head (CDK_STREAM out, size_t size, int type)
 
 
 static int
-pkt_write_head2 (CDK_STREAM out, int type)
+write_head_old (CDK_STREAM out, size_t size, int type)
 {
   int ctb;
-  
+  int rc;
+
   if (!out)
     return CDK_Inv_Value;
-  if (type < 0 || type > 63)
+  if (type < 0 || type > 16)
     return CDK_Inv_Packet;
-  ctb = 0x80 | (type << 2) | 3;
-  return stream_putc (out, ctb);
+  ctb = 0x80 | (type << 2);
+  if (!size)
+    ctb |= 3;
+  else if (size < 256)
+    ;
+  else if (size < 65536)
+    ctb |= 1;
+  else
+    ctb |= 2;
+  rc = stream_putc (out, ctb);
+  if (!size)
+    return rc;
+  if (!rc)
+    {
+      if (size < 256)
+        stream_putc (out, size);
+      else if (size < 65536)
+        write_16 (out, size);
+      else
+        write_32 (out, size);
+    }
+  return rc;
+}
+
+
+/* write special PGP2 packet header. PGP2 (wrongly) uses two byte header
+   length for signatures and keys even if the size is < 256. */
+static int
+pkt_write_head2 (CDK_STREAM out, size_t size, int type)
+{
+  int rc;
+  
+  rc = cdk_stream_putc (out, 0x80 | (type << 2) | 1);
+  if (!rc)
+    rc = cdk_stream_putc (out, size >> 8);
+  if (!rc)
+    rc = cdk_stream_putc (out, size & 0xff);
+  return rc;
 }
 
 
 static int
-write_encrypted (CDK_STREAM out, cdkPKT_encrypted * enc)
+pkt_write_head (CDK_STREAM out, int old_ctb, size_t size, int type)
+{
+  if (old_ctb)
+    return write_head_old (out, size, type);
+  return write_head_new (out, size, type);
+}
+
+
+static int
+write_encrypted (CDK_STREAM out, cdkPKT_encrypted * enc, int old_ctb)
 {
   int nbytes = 0;
   int rc = 0;
@@ -202,7 +249,7 @@ write_encrypted (CDK_STREAM out, cdkPKT_encrypted * enc)
     _cdk_log_debug ("** write encrypted packet %lu bytes\n", enc->len);
 
   nbytes = enc->len ? (enc->len + enc->extralen) : 0;
-  rc = pkt_write_head (out, nbytes, CDK_PKT_ENCRYPTED);
+  rc = pkt_write_head (out, old_ctb, nbytes, CDK_PKT_ENCRYPTED);
   /* rest of the packet is ciphertext */
 
   return rc;
@@ -225,7 +272,7 @@ write_encrypted_mdc (CDK_STREAM out, cdkPKT_encrypted * enc)
     _cdk_log_debug ("** write encrypted mdc packet %lu bytes\n", enc->len);
 
   nbytes = enc->len ? (enc->len + enc->extralen + 1) : 0;
-  rc = pkt_write_head (out, nbytes, CDK_PKT_ENCRYPTED_MDC);
+  rc = pkt_write_head (out, 0, nbytes, CDK_PKT_ENCRYPTED_MDC);
   if (!rc)
     rc = stream_putc (out, 1);	/* version */
   /* rest of the packet is ciphertext */
@@ -252,7 +299,7 @@ write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
   if (s2k->mode == 3)
     s2k_size++;
   size = 4 + s2k_size + ske->seskeylen;
-  rc = pkt_write_head (out, size, CDK_PKT_SYMKEY_ENC);
+  rc = pkt_write_head (out, 0, size, CDK_PKT_SYMKEY_ENC);
   if (!rc)
     rc = stream_putc (out, ske->version);
   if (!rc)
@@ -272,7 +319,7 @@ write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
 
 
 static int
-write_pubkey_enc (CDK_STREAM out, cdkPKT_pubkey_enc * pke)
+write_pubkey_enc (CDK_STREAM out, cdkPKT_pubkey_enc * pke, int old_ctb)
 {
   int size, nenc = 0;
   int rc = 0;
@@ -288,7 +335,7 @@ write_pubkey_enc (CDK_STREAM out, cdkPKT_pubkey_enc * pke)
 
   nenc = cdk_pk_get_nenc (pke->pubkey_algo);
   size = 10 + calc_mpi_size (pke->mpi, nenc);
-  rc = pkt_write_head (out, size, CDK_PKT_PUBKEY_ENC);
+  rc = pkt_write_head (out, old_ctb, size, CDK_PKT_PUBKEY_ENC);
   if (!rc)
     rc = stream_putc (out, 3);
   if (!rc)
@@ -328,14 +375,16 @@ static int
 subpkt_calc_size (CDK_SUBPKT s)
 {
   size_t nbytes;
-  
+
+  if (!s)
+    return 0;
   _cdk_subpkt_get_array (s, 1, &nbytes);
   return nbytes;
 }
 
 
 static int
-write_signature (CDK_STREAM out, cdkPKT_signature * sig)
+write_signature (CDK_STREAM out, cdkPKT_signature * sig, int old_ctb)
 {
   byte * buf;
   size_t nbytes;
@@ -357,7 +406,10 @@ write_signature (CDK_STREAM out, cdkPKT_signature * sig)
   if (sig->version < 4)
     {
       size = 19 + calc_mpi_size (sig->mpi, nsig);
-      rc = pkt_write_head (out, size, CDK_PKT_SIGNATURE);
+      if (is_RSA (sig->pubkey_algo))
+        rc = pkt_write_head2 (out, size, CDK_PKT_SIGNATURE);
+      else
+        rc = pkt_write_head (out, old_ctb, size, CDK_PKT_SIGNATURE);
       if (!rc)
 	rc = stream_putc (out, 3);
       if (!rc)
@@ -384,7 +436,7 @@ write_signature (CDK_STREAM out, cdkPKT_signature * sig)
       size = 10 + subpkt_calc_size (sig->hashed)
 	+ subpkt_calc_size (sig->unhashed)
 	+ calc_mpi_size (sig->mpi, nsig);
-      rc = pkt_write_head (out, size, CDK_PKT_SIGNATURE);
+      rc = pkt_write_head (out, 0, size, CDK_PKT_SIGNATURE);
       if (!rc)
 	rc = stream_putc (out, 4);
       if (!rc)
@@ -425,10 +477,11 @@ write_signature (CDK_STREAM out, cdkPKT_signature * sig)
 
 
 static int
-write_public_key (CDK_STREAM out, cdkPKT_public_key * pk, int is_subkey)
+write_public_key (CDK_STREAM out, cdkPKT_public_key * pk,
+                  int is_subkey, int old_ctb)
 {
   int rc = 0;
-  int size = 6, pkttype;
+  int size = 6, pkttype, ndays = 0;
   size_t npkey = 0;
 
   if (!out || !pk)
@@ -444,14 +497,23 @@ write_public_key (CDK_STREAM out, cdkPKT_public_key * pk, int is_subkey)
   npkey = cdk_pk_get_npkey (pk->pubkey_algo);
   if (pk->version < 4)
     size += 2; /* expire date */
+  if (is_subkey)
+    old_ctb = 0;
   size += calc_mpi_size (pk->mpi, npkey);
-  rc = pkt_write_head (out, size, pkttype);
+  if (old_ctb)
+    rc = pkt_write_head2 (out, size, pkttype);
+  else
+    rc = pkt_write_head (out, old_ctb, size, pkttype);
   if (!rc)
     rc = stream_putc (out, pk->version);
   if (!rc)
     rc = write_32 (out, pk->timestamp);
   if (!rc && pk->version < 4)
-    rc = write_16 (out, pk->expiredate);
+    {
+      if (pk->expiredate)
+        ndays = (u16)((pk->expiredate - pk->timestamp) / 86400L);
+      rc = write_16 (out, ndays);
+    }
   if (!rc)
     rc = stream_putc (out, pk->pubkey_algo);
   if (!rc)
@@ -461,10 +523,30 @@ write_public_key (CDK_STREAM out, cdkPKT_public_key * pk, int is_subkey)
 
 
 static int
-write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
+calc_s2k_size (cdkPKT_secret_key * sk)
+{
+  size_t nbytes = 0;
+  
+  if (!sk->is_protected)
+    return 0;
+  switch (sk->protect.s2k->mode)
+    {
+    case 0: nbytes = 2; break;
+    case 1:
+    case 3: nbytes = 11; break;
+    }
+  nbytes += sk->protect.ivlen;
+  return nbytes;
+}
+
+  
+static int
+write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk,
+                  int is_subkey, int old_ctb)
 {
   cdkPKT_public_key * pk = NULL;
-  size_t size = 0, npkey, nskey;
+  size_t size = 6, npkey, nskey;
+  int pkttype;
   int rc = 0;
 
   if (!out || !sk || !sk->pk)
@@ -479,8 +561,14 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
 
   npkey = cdk_pk_get_npkey (pk->pubkey_algo);
   nskey = cdk_pk_get_nskey (pk->pubkey_algo);
+  if (pk->version < 4)
+    size += 2;
+  size += (1 + calc_s2k_size (sk) + 2);
+  size += calc_mpi_size (pk->mpi, npkey);
+  size += calc_mpi_size (sk->mpi, nskey);
 
-  rc = pkt_write_head (out, size, CDK_PKT_SECRET_KEY);
+  pkttype = is_subkey? CDK_PKT_SECRET_SUBKEY : CDK_PKT_SECRET_KEY;
+  rc = pkt_write_head (out, old_ctb, size, pkttype);
   if (!rc)
     rc = stream_putc (out, pk->version);
   if (!rc)
@@ -519,7 +607,7 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
 	}
       rc = stream_write (out, sk->protect.iv, sk->protect.ivlen);
     }
-  if (!rc && sk->is_protected && pk->version > 4)
+  if (!rc && sk->is_protected && pk->version == 4)
     {
       if (sk->encdata && sk->enclen)
 	rc = stream_write (out, sk->encdata, sk->enclen);
@@ -529,7 +617,11 @@ write_secret_key (CDK_STREAM out, cdkPKT_secret_key * sk)
       if (!rc)
         rc = write_mpi_array (out, sk->mpi, nskey);
       if (!rc)
-	rc = write_16 (out, sk->csum);
+        {
+          if (!sk->csum)
+            sk->csum = _cdk_seckey_get_csum (sk);
+          rc = write_16 (out, sk->csum);
+        }
     }
 
   return rc;
@@ -547,7 +639,7 @@ write_compressed (CDK_STREAM out, cdkPKT_compressed * cd)
   if (DEBUG_PKT)
     _cdk_log_debug ("** write compressed packet\n");
 
-  rc = pkt_write_head2 (out, CDK_PKT_COMPRESSED);
+  rc = pkt_write_head (out, 1, 0, CDK_PKT_COMPRESSED);
   if (!rc)
     rc = stream_putc (out, cd->algorithm);
   return rc;
@@ -555,7 +647,7 @@ write_compressed (CDK_STREAM out, cdkPKT_compressed * cd)
 
 
 static int
-write_plaintext (CDK_STREAM out, cdkPKT_plaintext * pt)
+write_plaintext (CDK_STREAM out, cdkPKT_plaintext * pt, int old_ctb)
 {
   byte buf[4096];
   size_t size;
@@ -568,7 +660,7 @@ write_plaintext (CDK_STREAM out, cdkPKT_plaintext * pt)
     _cdk_log_debug ("** write plaintext packet\n");
 
   size = 6 + pt->namelen + pt->len;
-  rc = pkt_write_head (out, size, CDK_PKT_PLAINTEXT);
+  rc = pkt_write_head (out, old_ctb, size, CDK_PKT_PLAINTEXT);
   if (!rc)
     rc = stream_putc (out, pt->mode);
   if (!rc)
@@ -608,7 +700,7 @@ write_onepass_sig (CDK_STREAM out, cdkPKT_onepass_sig * sig)
     _cdk_log_debug ("** write one pass signature packet\n");
 
   size = 13;
-  rc = pkt_write_head (out, size, CDK_PKT_ONEPASS_SIG);
+  rc = pkt_write_head (out, 0, size, CDK_PKT_ONEPASS_SIG);
   if (!rc)
     rc = stream_putc (out, sig->version);
   if (!rc)
@@ -628,18 +720,18 @@ write_onepass_sig (CDK_STREAM out, cdkPKT_onepass_sig * sig)
 
 
 static int
-write_user_id (CDK_STREAM out, cdkPKT_user_id * id)
+write_user_id (CDK_STREAM out, cdkPKT_user_id * id, int old_ctb)
 {
   int rc = 0;
 
-  if (!out || !id)
+  if (!out || !id || !id->name)
     return CDK_Inv_Value;
 
   if (id->attrib_img)
     ;/* todo */
   else
     {
-      rc = pkt_write_head (out, id->len, CDK_PKT_USER_ID);
+      rc = pkt_write_head (out, old_ctb, id->len, CDK_PKT_USER_ID);
       if (!rc)
 	rc = stream_write (out, id->name, id->len);
     }
@@ -658,7 +750,7 @@ cdk_pkt_build (CDK_STREAM out, CDK_PACKET * pkt)
   switch (pkt->pkttype)
     {
     case CDK_PKT_PLAINTEXT:
-      rc = write_plaintext (out, pkt->pkt.plaintext);
+      rc = write_plaintext (out, pkt->pkt.plaintext, pkt->old_ctb);
       break;
     case CDK_PKT_ONEPASS_SIG:
       rc = write_onepass_sig (out, pkt->pkt.onepass_sig);
@@ -670,34 +762,34 @@ cdk_pkt_build (CDK_STREAM out, CDK_PACKET * pkt)
       rc = write_symkey_enc (out, pkt->pkt.symkey_enc);
       break;
     case CDK_PKT_ENCRYPTED:
-      rc = write_encrypted (out, pkt->pkt.encrypted);
+      rc = write_encrypted (out, pkt->pkt.encrypted, pkt->old_ctb);
       break;
     case CDK_PKT_ENCRYPTED_MDC:
       rc = write_encrypted_mdc (out, pkt->pkt.encrypted);
       break;
     case CDK_PKT_PUBKEY_ENC:
-      rc = write_pubkey_enc (out, pkt->pkt.pubkey_enc);
+      rc = write_pubkey_enc (out, pkt->pkt.pubkey_enc, pkt->old_ctb);
       break;
     case CDK_PKT_SIGNATURE:
-      rc = write_signature (out, pkt->pkt.signature);
+      rc = write_signature (out, pkt->pkt.signature, pkt->old_ctb);
       break;
     case CDK_PKT_PUBLIC_KEY:
-      rc = write_public_key (out, pkt->pkt.public_key, 0);
+      rc = write_public_key (out, pkt->pkt.public_key, 0, pkt->old_ctb);
       break;
     case CDK_PKT_PUBLIC_SUBKEY:
-      rc = write_public_key (out, pkt->pkt.public_key, 1);
+      rc = write_public_key (out, pkt->pkt.public_key, 1, pkt->old_ctb);
       break;
     case CDK_PKT_COMPRESSED:
       rc = write_compressed (out, pkt->pkt.compressed);
       break;
     case CDK_PKT_SECRET_KEY:
-      rc = write_secret_key (out, pkt->pkt.secret_key);
+      rc = write_secret_key (out, pkt->pkt.secret_key, 0, pkt->old_ctb);
       break;
     case CDK_PKT_SECRET_SUBKEY:
-      rc = write_secret_key (out, pkt->pkt.secret_key);
+      rc = write_secret_key (out, pkt->pkt.secret_key, 1, pkt->old_ctb);
       break;
     case CDK_PKT_USER_ID:
-      rc = write_user_id (out, pkt->pkt.user_id);
+      rc = write_user_id (out, pkt->pkt.user_id, pkt->old_ctb);
       break;
     default:
       rc = CDK_Inv_Packet;
