@@ -41,6 +41,7 @@ enum
   cSymmetric = 'c',
   cKeyList = 'k',
   cVerify = 'y',
+  cGenKey = 'w',
   cSigList = 'l',
   cEncr = 'e',
   cDecr = 'd',
@@ -57,6 +58,7 @@ enum
   cSignClear = 't',
   oArmor = 'p',
   oCompat = 'g',
+  oNoCompress = '0',
 };
 
 
@@ -92,6 +94,9 @@ int use_outname = 0;
 
 struct poptOption arguments[] =
 {
+  {"gen-key", cGenKey, POPT_ARG_NONE, 0, cGenKey, "Generate a key", 0},
+  {"no-compression", oNoCompress, POPT_ARG_NONE, 0, oNoCompress,
+   "Turns off compression", 0},
   {"compat", oCompat, POPT_ARG_NONE, 0, oCompat, "PGP compat mode", 0},
   {"verify", cVerify, POPT_ARG_STRING, &file, cVerify,
    "Verify a signature", 0},
@@ -151,6 +156,13 @@ callback (void * opaque, int type, const char * s)
 }
 
 
+static void
+progr_callback (void * opaque, unsigned off, unsigned len)
+{
+  printf ("off=%d len=%d\n", off, len);
+}
+
+
 static char *
 do_add_ext (const char * filename, const char * ext)
 {
@@ -174,7 +186,7 @@ do_remove_ext (const char * filename)
 
   if (!filename)
     return NULL;
-  if (!strstr (filename, ".gpg")
+  if (!strstr (filename, ".gpg") && !strstr (filename, ".pgp")
       && !strstr (filename, ".sig") && !strstr (filename, ".asc"))
     {
       output = cdk_calloc (1, strlen (filename) + 5);
@@ -268,6 +280,26 @@ algo_get_id (const char *algo, int what)
 }
 
 
+static char *
+get_keyvalid_char (cdkPKT_public_key * pk)
+{
+  static char flags[2];
+  memset (flags, 0, 2);
+  if (pk->has_expired)
+    strcat (flags, "e");
+  else
+    strcat (flags, " ");
+  if (pk->is_revoked)
+    strcat (flags, "r");
+  else
+    strcat (flags, " ");
+  if (!pk->has_expired && !pk->is_revoked)
+    strcat (flags, "!");
+  else
+    strcat (flags, " ");
+  return flags;
+}
+
 static void
 do_show_key (CDK_KBNODE pk, int with_sigs)
 {
@@ -280,20 +312,32 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
       if (pkt->pkttype == CDK_PKT_PUBLIC_KEY)
 	{
 	  cdkPKT_public_key * key = pkt->pkt.public_key;
-	  fprintf (stdout, "pub  %04d%c/%08lX %s\n",
+	  fprintf (stdout, "pub[%s]  %04d%c/%08lX %s",
+                   get_keyvalid_char (key),
 		   cdk_pk_get_nbits (key),
 		   key_get_letter (key->pubkey_algo),
 		   (unsigned long)cdk_pk_get_keyid (key, NULL),
 		   key_get_date (key->timestamp));
+          if (!key->expiredate)
+            fputc ('\n', stdout);
+          else
+            fprintf (stdout, "  [expires: %s]\n",
+                     key_get_date (key->expiredate));
 	}
       else if (pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
 	{
 	  cdkPKT_public_key * key = pkt->pkt.public_key;
-	  fprintf (stdout, "sub  %04d%c/%08lX %s\n",
+	  fprintf (stdout, "sub[%s]  %04d%c/%08lX %s",
+                   get_keyvalid_char (key),
 		   cdk_pk_get_nbits (key),
 		   key_get_letter (key->pubkey_algo),
 		   (unsigned long)cdk_pk_get_keyid (key, NULL),
 		   key_get_date (key->timestamp));
+          if (!key->expiredate)
+            fputc ('\n', stdout);
+          else
+            fprintf (stdout, "  [expires: %s]\n",
+                     key_get_date (key->expiredate));
 	}
       else if (pkt->pkttype == CDK_PKT_USER_ID)
 	{
@@ -609,6 +653,36 @@ print_verify_stats (CDK_HD hd)
 }
 
 
+int
+genkey (void)
+{
+  int c, algo = 0, rc =0;
+  CDK_KEYGEN_CTX hd;
+  
+  printf ("(1) DSA (sign only)\n");
+  c = fgetc (stdin);
+  switch (c)
+    {
+    case '1': algo = GCRY_PK_DSA; break;
+    default: printf ("invalid pk algorithm\n"); break;
+    }
+  if (!algo)
+    return CDK_Inv_Algo;
+  cdk_salloc (1);
+  cdk_keygen_new (&hd);
+  cdk_keygen_set_algo_info (hd, 0, algo, 768);
+  cdk_keygen_set_algo_info (hd, 1, GCRY_PK_ELG_E, 768);
+  cdk_keygen_set_name (hd, "Otto Bismarck");
+  cdk_keygen_set_mdc_feature (hd, 1);
+  cdk_keygen_set_expire_date (hd, 1040820537);
+  rc = cdk_keygen_start (hd);
+  if (!rc)
+    rc = cdk_keygen_save (hd, "pub.test", "sec.test");
+  cdk_keygen_free (hd);
+  return rc;
+}
+
+
 int
 main (int argc, char ** argv)
 {
@@ -626,9 +700,12 @@ main (int argc, char ** argv)
   /*cdk_keydb_add_resource ("secring.gpg", 1);*/
   /*cdk_keydb_add_resource ("sec2.gpg", 1);*/
   /*cdk_passphrase_cb_set (get_passphrase);*/
+  /*cdk_keydb_add_resource ("/home/twoaday/.pgp/pubring.pgp", 0);
+    cdk_keydb_add_resource ("/home/twoaday/.pgp/secring.pgp", 1);*/
 
   cdk_handle_new (&hd);
   cdk_handle_set_callback (hd, callback, NULL);
+  cdk_set_progress_handler (progr_callback, NULL);
   opt_ctx = poptGetContext ("pred", argc, argv, arguments, 0);
   while ((c = poptGetNextOpt (opt_ctx)) >= 0)
     {
@@ -638,6 +715,10 @@ main (int argc, char ** argv)
 	  verbose++;
 	  break;
 
+        case oNoCompress:
+          cdk_handle_set_compress (hd, 0);
+          break;
+          
         case 'p':
           cdk_handle_set_armor (hd, 1);
           break;
@@ -649,7 +730,7 @@ main (int argc, char ** argv)
 	  break;
 
         case 'g':
-          cdk_handle_set_compat (hd, 1);
+          cdk_handle_set_compat (hd, -1);
           break;
 
 	case 'h':
@@ -790,6 +871,12 @@ main (int argc, char ** argv)
 		     file, cdk_strerror (rc));
 	  break;
 
+        case 'w':
+          rc = genkey ();
+          if (rc)
+            printf ("key generation failed `%s'\n", cdk_strerror (rc));
+          break;
+          
         case 's':
           if (!remusr)
             {
@@ -799,7 +886,7 @@ main (int argc, char ** argv)
           output = do_add_ext (file, ".sig");
           if (verbose)
             fprintf (stdout, "sign `%s' -> %s\n", file, output);
-          rc = cdk_file_sign (hd, remusr, NULL, file, output, 0, 0);
+          rc = cdk_file_sign (hd, remusr, NULL, file, output, 1, 0);
           if (rc)
             fprintf (stderr, "minpg: sign file `%s' failed: %s\n",
                      file, cdk_strerror (rc));
@@ -839,4 +926,11 @@ main (int argc, char ** argv)
   cdk_free (output);
   return 0;
 }
+#else
+int
+main (int argc, char ** argv)
+{
+  printf ("MinPG example is not available.\n");
+  return 0;
+}
 #endif /* HAVE_LIBPOPT */
