@@ -502,6 +502,33 @@ test_stream_kbnode (const char * file)
 }
 
 
+#if TEST
+int classify_data (const byte * buf, size_t len);
+     
+void
+test_classify_function (void)
+{
+  int i;
+  const char * s;
+  const char * kid [] =
+  {
+    "0xBF3DF9B4",
+    "BF3DF9B4",
+    "bf3df9b4",
+    "ED4681C9BF3DF9B4",
+    "0xED4681C9BF3DF9B4",
+    "timo",
+    "ts@winpt.org",
+    NULL
+  };
+  for (i = 0, s = kid[i]; kid[i]; i++)
+    printf ("classify_data `%s' = %d\n", kid[i],
+            classify_data (kid[i], strlen (kid[i])));
+  
+} 
+#endif
+
+
 int
 main (int argc, char **argv)
 {
